/*******************************************************************
 *
 *	File:		ColorPicker.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __COLOR_PICKER_H
#define __COLOR_PICKER_H

#include <e32def.h>
#include <coecntrl.h>
#include <eikcmbut.h>

#include "EGnuChessMiscCtls.h"

const TInt KCustomColorsHor	= 5;
const TInt KCustomColorsVer	= 3;
const TInt KCustomColors	= KCustomColorsHor * KCustomColorsVer;

class TCustomColors
{
public:
	TCustomColors(){Defaults();}
	TRgb iColors[KCustomColors];

	void Defaults();
};

inline void TCustomColors::Defaults()
{
	for(TInt i=0 ; i<KCustomColors ; i++)
		iColors[i] = KRgbWhite;
}



/************************************************
 *
 * 
 * Color chooser
 *
 ************************************************/

class CEikLabel;

class TCustomColors;
class CColorChoserColorGraph;
class CColorChoserLumBar;
class CColorChoserLumSlider;
class CColorChoserColorPreview;

class CColorChooserCtl : public CCoeControlContainer, public MCoeControlObserver
{
public:
	~CColorChooserCtl();
	CColorChooserCtl();
	void ConstructL();

	void SetColor(const TRgb& aColor, TBool aRedraw=EFalse);
	TRgb Color();

	void SetCustomColors(const TCustomColors& aCustomColors);
	void CustomColors(TCustomColors& aCustomColors);

protected:
	virtual void SizeChanged();
	virtual void DoLayoutChildren();
	virtual void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);
	void DoLayoutCustomColorButtons();

	void SetColorTextL(TBool aRedraw = ETrue);
	void SetPreviewColor(TBool aRedraw = ETrue);

	void HueAndSatChangedL(TInt aHue, TInt aSat);
	void LumChangedL(TInt aLum);
	void ColorChangedL(TInt aHue, TInt aSat, TInt aLum);

	CColorChoserColorGraph*		iColorChoserColorGraph;
	CColorChoserLumBar*			iColorChoserLumBar;
	CColorChoserLumSlider*		iColorChoserLumSlider;
	CColorChoserColorPreview*	iColorChoserColorPreview;
	CColorChoserColorPreview*	iCustomColors[KCustomColors];
	CColorChoserColorPreview*	iFocussedPreviewButton;

	void FocusToPreviewButton(CColorChoserColorPreview* aControl);
	TRect FocussedRect(CCoeControl* aControl) const;
	void DrawFocussedRect(CBitmapContext& aBitmapContext, const TRect& aRect) const;
	void Draw(const TRect& aRect) const;

//	CEikLabel*			iLabel;
	CBitmappedLabel*	iLabel;
	TRgb			iColor;
};


#endif			/* __COLOR_PICKER_H */