/*******************************************************************
 *
 *	File:		EGnuChessAppUi.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_APPUI_H
#define __E_GNU_CHESS_APPUI_H

#include <qikappui.h>
#include "EGnuChessDocument.h"
#include "EGnuChessModelObserver.h"

class CEGnuChessViewBase;
class CEGnuChessViewCtl;
class CChessModel;
class CEikButtonGroupContainer;


class CToolbarIconFlasher : public CBase
{
public:
	~CToolbarIconFlasher();
	static CToolbarIconFlasher* NewL();
	void Start(CEikButtonGroupContainer* aToolbar, TInt aCommand);
protected:
	CToolbarIconFlasher(){}
	void ConstructL();
	static TInt PeriodicCallback(TAny* aAny);
	void DoFlash();
	CEikButtonGroupContainer*	iToolbar;
	TInt						iCommand;
	TInt						iStep;
	CPeriodic*					iPeriodic;
};



class CEGnuChessAppUi : public CQikAppUi, public MEGnuChessModelGameObserver
{
public:
	~CEGnuChessAppUi();
	void ConstructL();

	CEGnuChessModel& Model() const { return STATIC_CAST(CEGnuChessDocument*,iDocument)->Model(); }

protected:		// implements MEGnuChessModelGameObserver
	virtual void UserThinksL();
	virtual void MachineThinksL();
	virtual void DisplayMessageL(const TDesC& /*aMessage*/){};
	virtual void CommentPopUpL(const TDesC& aTitle, const TDesC& aComment);
	virtual void CommentPopDown();
	virtual void EditCommentPopUpL(TInt aIndex, const TDesC& aTitle, const TDesC& aComment);
	virtual void ModeHighlightL();

protected:
	void HandleCommandL(TInt aCommand);
	void SaveGameL();
	void AboutL();
	void SaveFile();
	void LoadGameL();
	void BoardOptionsL();
	void ClockOptionsL();
	void EditTagsL();
	void EditCommentsL();
	void SelectGameL();

	void ResetUI();
	void EnableSelectGame(TBool aEnable);

	void HandleApplicationSpecificEventL(TInt aType, const TWsEvent& aEvent);
	void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);

	void DynInitFileMenuPaneL(CEikMenuPane* aMenuPane);
	void DynInitModeMenuPaneL(CEikMenuPane* aMenuPane);
	void DynInitActionMenuPaneL(CEikMenuPane* aMenuPane);
	void DynInitStepMenuPaneL(CEikMenuPane* aMenuPane);
	void DynInitOptionsMenuPaneL(CEikMenuPane* aMenuPane);
	void DynInitFcMenuPaneL(CEikMenuPane* aMenuPane);

	void UpdatePauseMenuPaneL(CEikMenuPane* aMenuPane);
	void UpdateModeMenuPaneL(CEikMenuPane* aMenuPane);

	void SendCommandL();
	void EditCraftyRcL();

	void GamePausedOrResume(TBool aIsPaused);
	
protected:
    CEGnuChessViewBase*		iFOView;
    CEGnuChessViewBase*		iFCView;
    CEGnuChessViewCtl*		iEGnuChessViewCtl;

	void ActivateChessViewL();


	TInt		iModeNowChecked;
	TInt		iPrevPausing;
	TBool		iUserInitiatedComment;
	TInt		iGameNo;

	CEikButtonGroupContainer*	iToolbar;
	CToolbarIconFlasher*		iToolbarIconFlasher;
	CPeriodic*					iSystemActivityTimer;
};


#endif // __E_GNU_CHESS_APPUI_H




