/*******************************************************************
 *
 *	File:		EGnuChessDialogs.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_DIALOGS_H
#define __E_GNU_CHESS_DIALOGS_H

#include <eikdialg.h>
#include "EGnuChessOptions.h"
#include "EGnuChessBoardCtl.h"

#define BuildCategoriesListL() BuildCategoriesListL();friend class CSaveDocumentDialog;
#include <QikMediaFolderMonitor.h>

class CEikChoiceList;
class CEikEdwin;


class CSaveDocumentDialog : public CEikDialog, protected MQikMediaFolderMonitorObserver
{
public:
	~CSaveDocumentDialog();
	CSaveDocumentDialog(const TDesC& aDocName, const TDesC& aDocTitle, TFileName& aFileName);

protected:			// implements MQikMediaFolderMonitorObserver
	virtual void FolderChangeL(const TDriveNumber& /*aDrive*/, const TDesC& /*aFolder*/,  const TBool /*aUpdateNeeded*/){};
	virtual void FolderChangeToAllL(const TBool /*aUpdateNeeded*/){};

protected:
	virtual TBool OkToExitL(TInt aButtonId);
	virtual void PreLayoutDynInitL();
	virtual void PostLayoutDynInitL();
	virtual void HandleControlStateChangeL(TInt aControlId);

	void UpdateLocation();
	void UpdateFolders();
	TBool EditFolders();
	TBool GetFileName();
	void MakeUniqueName(const TDesC& aDocName, TDes& aNumberedDocName, TInt aNum);


	TFileName		iDocName;
	const TPtrC		iDocTitle;
	TFileName&		iFileName;

	TBool			iShowExternal;
	TBool			iHasExternalDir;

	TDriveNumber	iExternalDrive;

	CQikMediaFileFolderUtils*		iQikMediaFileFolderUtils;
	CDesCArray*						iEChessMimeArray;

	CEikChoiceList*		iLocationChoiceList;
	CEikChoiceList*		iFoldersChoiceList;
	CEikEdwin*			iNameEdwin;
};


/**********************************************
 *
 * Board settings
 *
 **********************************************/
class TBoardSettings;

class CBoardSettingsDialog : public CEikDialog
{
public:
	~CBoardSettingsDialog();
	CBoardSettingsDialog(TBoardSettings& aBoardSettings, TCustomColors& TCustomColors);

protected:
	TBool OkToExitL(TInt aButtonId);
	SEikControlInfo CreateCustomControlL(TInt aControlType);
	void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	void ChangeColorL(TRgb& aColor);
	void ChangePatternL();
	void HandleControlStateChangeL(TInt aControlId);
	void UpdateControls(TBool aRedraw=EFalse);

	TBoardSettings&		iBoardSettings;
	TCustomColors&		iCustomColors;
	TChessBitmaps		iChessBitmaps;
};


/**********************************************
 *
 * Choose color dialog
 *
 **********************************************/

//class CColorChooserCtl;
class TRgb;
class TCustomColors;

class CChooseColorDialog : public CEikDialog
{
public:
	CChooseColorDialog(TRgb& aColor, TCustomColors& aCustomColor);

protected:
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	SEikControlInfo CreateCustomControlL(TInt aControlType);
	void SetSizeAndPosition(const TSize& aSize);

//	CColorChooserCtl*	iColorChooserCtl;
	TRgb&				iColor;
	TCustomColors&		iCustomColors;
};



/**********************************************
 *
 * Choose pattern dialog
 *
 **********************************************/

class CPatternChooserCtl;

class CPatternDialog : public CEikDialog
{
public:
	CPatternDialog(TInt& aPattern, const TBoardSettings& aBoardSettings);

protected:
	TBool OkToExitL(TInt aButtonId);
	void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	SEikControlInfo CreateCustomControlL(TInt aControlType);

	const TBoardSettings& iBoardSettings;
	CPatternChooserCtl*	iPatternChooserCtl;
	TInt&				iPattern;
};


/**********************************************
 *
 * Edit dialog
 *
 **********************************************/

class CEditDialog : public CEikDialog
{
public:
	CEditDialog();

protected:
	void PostLayoutDynInitL();

	void Text8ToEdwin(const TText8* aText8, TBool aUpdateEdwin = EFalse);	// sets iText
	void Text8FromEdwin();						// sets iText8

	TBuf<KMaxEditSize>		iText;
	char					iText8[KMaxEditSize];
};


/**********************************************
 *
 * Edit tags dialog
 *
 **********************************************/

class CEditTagsDialog : public CEditDialog
{
public:
	static void RunLD();

protected:
	CEditTagsDialog();

protected:
	TBool OkToExitL(TInt aButtonId);
	void SetTagsToEdwin();
	void GetTags8FromEdwin();
	TBool SaveTags();
};


/**********************************************
 *
 * Edit comments dialog
 *
 **********************************************/

class CEditCommentsDialog : public CEditDialog
{
public:
	static void RunLD(TInt aIndex, const TDesC& aTitle, const TDesC& aComment);

protected:
	CEditCommentsDialog(TInt aIndex, const TDesC& aComment);

protected:
	TBool OkToExitL(TInt aButtonId);
	void SetCommentsToEdwin();
	void GetComments8FromEdwin();
	void SaveComments();

	TInt iIndex;
};




/**********************************************
 *
 * Select game dialog
 *
 **********************************************/


class CSelectGameDialog : public CEikDialog
{
public:
	static TInt RunLD(TInt& aGameNo, const TDesC& aTitle);

protected:
	CSelectGameDialog(TInt& aGameNo);
	virtual void PreLayoutDynInitL();

protected:
	TBool OkToExitL(TInt aButtonId);

	TInt& iGameNo;
};



/**********************************************
 *
 * Clock settings dialog
 *
 **********************************************/

class CClockSettingsDialog : public CEikDialog
{
public:
	CClockSettingsDialog(TClockSettings& aClockSettings);

protected:
	void PostLayoutDynInitL();
	void UpdateControlsL(TBool aRedraw);
	void HandleControlStateChangeL(TInt aControlId);
	TBool ValidateDataL();

	TBool GetEdwinInt(TInt& aInt, TInt aEdwinRes);
	void SetEdwinInt(TInt aInt, TInt aEdwinRes);

	TBool OkToExitL(TInt aButtonId);
	TClockSettings&		iClockSettings;
};


/**********************************************
 *
 * COneLineEdit dialog
 *
 **********************************************/

class COneLineEdit : public CEditDialog
{
public:
	COneLineEdit(TDes& aLine);

protected:
	TBool OkToExitL(TInt aButtonId);
	TDes&	iLine;
};


/**********************************************
 *
 * CAboutDialog dialog
 *
 **********************************************/

class CAboutDialog : public CEikDialog
{
public:
	CAboutDialog(){}
protected: // framework
    void PreLayoutDynInitL();
};


/**********************************************
 *
 * Edit text file dialog
 *
 **********************************************/

class CEditTextFileDialog : public CEditDialog
{
public:
	static void RunLD(const TDesC& aFileName, const TDesC& aDefaultFile, TInt aTitleResId);

protected:
	CEditTextFileDialog(const TDesC& aFileName, const TDesC& aDefaultFile);

protected:
	TBool OkToExitL(TInt aButtonId);
	void ReadTextFile(const TDesC& aFileName, TBool aUpdateEdwin = EFalse);
	TBool StoreTextFile(const TDesC& aFileName);

	TFileName	iFileName;
	TFileName	iDefaultFileName;
};



#endif			/* __E_GNU_CHESS_DIALOGS_H */