/*******************************************************************
 *
 *	File:		EGnuChessEngConnection.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_ENG_CONNECTION_H
#define __E_GNU_CHESS_ENG_CONNECTION_H

#include <e32std.h>
#include <es_sock.h>

const TInt KMaxRecvBuf = 512;

class MEGnuChessEngDataProcessor
{
public:
	virtual void ProcessData(const TDesC8& aData) = 0;
};


class CEGnuChessEngConnection : public CActive
{
public:
	~CEGnuChessEngConnection();
	static CEGnuChessEngConnection* NewL();
	void SetEngineDataProcessor(MEGnuChessEngDataProcessor* aEGnuChessEngDataProcessor);
	TInt EngineWriteData(const TDesC8& aData);

protected:
	CEGnuChessEngConnection();
	void ConstructL();
	void StartAndWaitForEngineToConnectL();

	void EngineStop();
	void EngineStartL();
	void EngineReceiveData();
	void EngineProcessData();

protected:		// from CActive
	void RunL();
	void DoCancel();

protected:
	TThreadId		iChessEngineThread;
	RSocket			iServerSocket;
	RSocket			iSocket;
	RSocketServ		iSockSrvSession;

	TBuf8<KMaxRecvBuf>	iRawRecvBuf;
	TSockXfrLength		iRawRecvBufLen;
	TBuf8<KMaxRecvBuf>	iRecvBuf;

	MEGnuChessEngDataProcessor* iEGnuChessEngDataProcessor;
};


#endif			/* __E_GNU_CHESS_ENG_CONNECTION_H */
