/*******************************************************************
 *
 *	File:		EGnuChessFrontendProcessor.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_FRONTEND_PROCESSOR_H
#define __E_GNU_CHESS_FRONTEND_PROCESSOR_H

#include "winboard.h"

typedef ChessSquare TBoard[BOARD_SIZE][BOARD_SIZE];

class TSquare
{
public:
	TSquare() : iCol(-1), iRow(-1) {};
	TSquare(TInt aRow, TInt aCol);
	TSquare(const TSquare& aSquare);
	TInt iCol;
	TInt iRow;
	TBool operator ==(const TSquare& aSquare) const;
	TBool operator !=(const TSquare& aSquare) const;
	TBool IsEmpty() const;
	void Empty();
	void Normalize();
};


inline TSquare::TSquare(TInt aRow, TInt aCol) : iCol(aCol), iRow(aRow)
{
}

inline TSquare::TSquare(const TSquare& aSquare) : iCol(aSquare.iCol), iRow(aSquare.iRow)
{
}

inline TBool TSquare::operator ==(const TSquare& aSquare) const
{
	return ((iRow == aSquare.iRow) && (iCol == aSquare.iCol));
}

inline TBool TSquare::operator !=(const TSquare& aSquare) const
{
	return !operator ==(aSquare);
}

inline TBool TSquare::IsEmpty() const
{
	return ((iRow == -1) && (iCol == -1));
}

inline void TSquare::Empty()
{
	iRow = iCol = -1;
}

inline void TSquare::Normalize()
{
	if ( (iCol<0) || (iCol>7) || (iRow<0) || (iRow>7) )
		Empty();
}


class MFrontendProcessor
{
public:
	virtual void ModeHighlightL() = 0;
	virtual void ResetFrontEndL() = 0;
	virtual void ClearHighlightsL() = 0;
	virtual void ClearPremoveHighlightsL() = 0;
	virtual void DrawPositionL(TInt aFullRedraw, TBoard* aBoard) = 0;
	virtual void SetHighlightsL(const TSquare& aSquareFrom, const TSquare& aSquareTo) = 0;
	virtual void UserThinksL() = 0;
	virtual void MachineThinksL() = 0;
	virtual void DisplayMessageL(const TDesC& aMessage) = 0;
	virtual void UpdateTimeClockL(TBool aWhite, const TDesC& aTimeString, TBool aHighlight) = 0;
	virtual void CommentPopUpL(const TDesC& aTitle, const TDesC& aComment) = 0;
	virtual void CommentPopDown() = 0;
	virtual void EditCommentPopUpL(TInt aIndex, const TDesC& aTitle, const TDesC& aText) = 0;
};

void SetFrontendProcessor(MFrontendProcessor* aFrontendProcessor);


#endif			/* __E_GNU_CHESS_FRONTEND_PROCESSOR_H */
