/*******************************************************************
 *
 *	File:		EGnuChessMiscCtls.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_MISC_CTLS_H
#define __E_GNU_CHESS_MISC_CTLS_H

#include <coecntrl.h>
#include <eikcmbut.h>

/*****************************************************
 *
 * CoeContontrolContainer:
 * - maintains list of children 
 * - clears it's entire Rect() on redraw except the area
 *   of those children that indicate they don't want to
 *   be blanked.
 *
 *****************************************************/

class CCoeControlContainer : public CCoeControl
{
public:
	~CCoeControlContainer();
	void ConstructL();
	void AddChildL(CCoeControl* aChild, TBool aBlankChild=EFalse);
	void RemoveChild(CCoeControl* aChild);
	void LayoutChildren();
	void DrawChildNow(CCoeControl* aChild);
	void DrawAreaNow(const TRect& aRect);

protected:
	virtual void DoLayoutChildren() = 0;
	void DestroyChildren();
	TBool ChildNeedsBlanking(CCoeControl* aChild);

protected:
	CCoeControl* ComponentControl(TInt aIndex) const;
	TInt CountComponentControls() const;
	void Draw(const TRect& aRect) const;
	void SetRegion();

	CArrayPtrFlat<CCoeControl>*		iChildren;
	CArrayPtrFlat<CCoeControl>*		iDontBlankChildren;
	TInt							iNoOfChildren;
	RRegion							iBlankChildrenRegion;
};



/*****************************************************
 *
 * CBitmappedCoeControl:
 * - allows the control to draw to an offscreen bitmap
 *   which get blitted to screen
 * - control should call DrawControl() when it wants
 *   to redraw itself
 * - CBitmappedCoeControl will gracefully cope with the fact
 *   that the offline bitmap cannot be allocated due to
 *   lack of memory
 *
 *****************************************************/

class CBitmappedCoeControl : public CCoeControl
{
public:
	~CBitmappedCoeControl() {DestroyBitmap();}
	void DrawControl();
protected:
	virtual void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const = 0;
	void SizeChanged();
	void DestroyBitmap();
	void CreateBitmapL(const TSize& aSize);
	void Draw(const TRect& aRect) const;
protected:
	CFbsBitGc*			iBitmapContext;
	CFbsBitmap*			iBitmap;
	CFbsBitmapDevice*	iBitmapDevice;
};


class CBitmappedLabel : public CBitmappedCoeControl
{
public:
	~CBitmappedLabel();
	CBitmappedLabel();

	void ReleaseUserFont();
	TInt UseFont(const TDesC& aFontName, TInt aFontSize);
	void SetTextL(const TDesC& aText);
	void SetBgColor(const TRgb& aColor){iBgColor=aColor;}

	const CFont* Font() const;
protected:
	virtual void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const;
	HBufC*			iText;
	const CFont*	iSystemFont;
	CFont*			iUserFont;
	TRgb			iBgColor;
};

#endif			/* __E_GNU_CHESS_MISC_CTLS_H */