/*******************************************************************
 *
 *	File:		EGnuChessModel.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_MODEL_H
#define __E_GNU_CHESS_MODEL_H

#include <e32base.h>	// class CBase

#include "winboard.h"
#include "EGnuChessFrontendProcessor.h"
#include "EGnuChessApp.hrh"		// for KMaxTagsSize

#include "EGnuChessOptions.h"

class CEGnuChessDocument;
class MEGnuChessModelBoardObserver;
class MEGnuChessModelGameObserver;
class MEGnuChessModelTimeObserver;

const TInt KMaxObservers = 5;

typedef TBuf8<KMaxFileName> TFileName8;



class CEGnuChessModel : public CBase, public MFrontendProcessor
{

public:
	static CEGnuChessModel* NewL(CEGnuChessDocument& aDocument);
	~CEGnuChessModel();
	void AddModelBoardObserver(MEGnuChessModelBoardObserver& aModelBoardObserver);
	void AddModelGameObserver(MEGnuChessModelGameObserver& aModelGameObserver);
	void SetModelTimeObserver(MEGnuChessModelTimeObserver& aModelTimeObserver);
	void StartBackEnd();

	TBool IsOKToStartUserMove(const TSquare& aSquare);
	TBool IsPromotion(const TSquare& aSquare);
	void UserMoveEvent(const TSquare& aSquare, TInt aPromoChar);
	void SetHighlight(const TSquare& aSquare);
	void SetTimeControl();

public:
	void SaveGame();
	void SelectedGame(TInt aGameNo, TBool aUseList);
	void LoadGame(const TDesC& aFileName, TInt& aNoOfGames);
	TFileName		iFileName;

protected:
	CEGnuChessModel(CEGnuChessDocument& aDocument);
	void ConstructL();
	void InitAppData();

	TFileName8		iFileName8;
	FILE*			iFile;

public:
	TSquare	iHighlights[2];
	TSquare	iPRemoveHighlights[2];
	TBoard	iBoard;
	TBoard	iPrevBoard;
	TBool	iUserThinks;

public:			// implements MGnuChessFrontendProcessor
	virtual void ModeHighlightL();
	virtual void ResetFrontEndL();
	virtual void ClearHighlightsL();
	virtual void ClearPremoveHighlightsL();
	virtual void DrawPositionL(TInt aFullRedraw, Board* aBoard);
	virtual void SetHighlightsL(const TSquare& aSquareFrom, const TSquare& aSquareTo);
	virtual void UserThinksL();
	virtual void MachineThinksL();
	virtual void DisplayMessageL(const TDesC& aMessage);
	virtual void UpdateTimeClockL(TBool aWhite, const TDesC& aTimeString, TBool aHighlight);
	virtual void CommentPopUpL(const TDesC& aTitle, const TDesC& aComment);
	virtual void CommentPopDown();
	virtual void EditCommentPopUpL(TInt aIndex, const TDesC& aTitle, const TDesC& aText);

	const TEGnuChessOptions& ChessOptions();
	void SetChessOptions(const TEGnuChessOptions& aOptions);

protected:
	CEGnuChessDocument&				iDocument;
	MEGnuChessModelBoardObserver*	iModelBoardObservers[KMaxObservers];
	MEGnuChessModelGameObserver*	iModelGameObservers[KMaxObservers];
	MEGnuChessModelTimeObserver*	iModelTimeObserver;


	TEGnuChessOptions	iEGnuChessOptions;

public:		// accessors from the C-based frontend

};

#endif			/* __E_GNU_CHESS_MODEL_H */
