/*******************************************************************
 *
 *	File:		EGnuChessOptions.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __E_GNU_CHESS_OPTIONS_H
#define __E_GNU_CHESS_OPTIONS_H

#include <gdi.h>			// for TRgb
#include "ColorPicker.h"	// for TCustomColors

class TBoardSettings
{
public:
	TBoardSettings(){Defaults();}

	TRgb		iColorLightSquare;
	TRgb		iColorDarkSquare;
	TRgb		iColorWhitePiece;
	TRgb		iColorBlackPiece;
	TRgb		iColorHighlight;
	TInt		iPattern;
	
	void Defaults();
};


inline void TBoardSettings::Defaults()
{
	iColorLightSquare	= TRgb(200, 195, 101);
	iColorDarkSquare	= TRgb(119, 162, 109);
	iColorWhitePiece	= KRgbWhite;
	iColorBlackPiece	= KRgbBlack;
	iColorHighlight		= KRgbRed;
	iPattern			= 0;
}


class TClockSettings
{
public:
	enum
	{
		EClockTypeConventional = 0,
		EClockTypeIncremental
	};
	TClockSettings(){Defaults();}
	void Defaults();

	TInt		iClockType;
	TInt		iConventionalNoOfMoves;
	TBuf<16>	iConventionalTime;
	TBuf<16>	iIncrementalInitialTime;
	TInt		iIncrementalAdditionalTimePerMove;
};


inline void TClockSettings::Defaults()
{
	iClockType = EClockTypeConventional;
	iConventionalNoOfMoves = 40;
	iConventionalTime = _L("5");
	iIncrementalInitialTime = _L("5");
	iIncrementalAdditionalTimePerMove = 0;
}

class TEGnuChessOptions
{
public:
	TBoardSettings		iBoardSettings;
	TCustomColors		iCustomColors;
	TClockSettings		iClockSettings;

	TBool Save(RFs& aFs);
	void Load(RFs& aFs);
};


#define KOptionsFileName	_L("c:\\system\\data\\echess.ini")

inline void TEGnuChessOptions::Load(RFs& aFs)
{
	RFile	file;
	TInt err = file.Open(aFs, KOptionsFileName, EFileRead);
	if (!err)
	{
		TPckg<TEGnuChessOptions>	buf(*this);
		err = file.Read(buf);
		file.Close();
	}
	else if (err == KErrNotFound)
	{
		err = KErrNone;
	}

	if (err)
	{
		_LIT(KLoadErr, "Settings could not be loaded");
		User::InfoPrint(KLoadErr);
	}
}


inline TBool TEGnuChessOptions::Save(RFs& aFs)
{
	RFile	file;
	TInt err = file.Open(aFs, KOptionsFileName, EFileWrite);
	if (err)
		err = file.Create(aFs, KOptionsFileName, EFileWrite);
	if (!err)
	{
		TPckgC<TEGnuChessOptions>	buf(*this);
		err = file.Write(buf);
		file.Close();
	}
	TBool	success = (err == KErrNone);

	_LIT(KNoSaveErr, "Settings saved");
	_LIT(KSaveErr, "Settings could not be saved");
	if (success)
		User::InfoPrint(KNoSaveErr);
	else
		User::InfoPrint(KSaveErr);
	
	return success;
}


#endif			/* __E_GNU_CHESS_OPTIONS_H */
