/*******************************************************************
 *
 *	File:		EGnuChessView.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#ifndef __E_GNU_CHESS_VIEW_H
#define __E_GNU_CHESS_VIEW_H

#include <eikon.hrh>
#include <coecntrl.h>
#include <coeccntx.h>

#include "EGnuChessModelObserver.h"
#include "EGnuChessMiscCtls.h"

class CEGnuChessModel;
class CEGnuChessBoardControl;
class CEikLabel;
class CEGnuChessViewCtl;
class TBoardSettings;

const TInt KNoOfCoeChildren = 4;

class CEGnuChessViewBase :	public CBase, public MCoeView
{
public:
	CEGnuChessViewBase(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl);
	~CEGnuChessViewBase();

protected:		// implements MCoeView
	virtual void ViewDeactivated();
	virtual void ViewConstructL();
	virtual void ViewActivatedL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/);

	CEGnuChessViewCtl&	iEGnuChessViewCtl;
	CEGnuChessModel&	iModel;
};

class CEGnuChessFOView :	public CEGnuChessViewBase
{
public:
	CEGnuChessFOView(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl);
	~CEGnuChessFOView();
	virtual TVwsViewId ViewId() const;
	virtual TVwsViewIdAndMessage ViewScreenDeviceChangedL();
	virtual TBool ViewScreenModeCompatible(TInt aScreenMode);
	virtual void ViewActivatedL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/);
};

class CEGnuChessFCView :	public CEGnuChessViewBase
{
public:
	CEGnuChessFCView(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl);
	~CEGnuChessFCView();
	virtual TVwsViewId ViewId() const;
	virtual TVwsViewIdAndMessage ViewScreenDeviceChangedL();
	virtual TBool ViewScreenModeCompatible(TInt aScreenMode);
	virtual void ViewActivatedL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/);
};


/*
 * We use the same CEGnuChessViewCtl object in FO & FC because it can render
 * the chessboard in both modes. Typically you'd implement a view control
 * for each mode.
 */

class CEGnuChessViewCtl : public CCoeControlContainer, public MCoeControlBrushContext,
	public MEGnuChessModelGameObserver, public MEGnuChessModelTimeObserver
{
public:
	static CEGnuChessViewCtl* NewL(CEGnuChessModel& aModel);
	~CEGnuChessViewCtl();
	void ChangeLayout(TBool aSmall);
	void Activated();
	void Deactivated();
	void SetBoardSettings(const TBoardSettings& aBoardSettings);
	
public:			// implements MEGnuChessModelInfoObserver
	virtual void DisplayMessageL(const TDesC& aMessage);
	virtual void UserThinksL(){};
	virtual void MachineThinksL(){};
	virtual void CommentPopUpL(const TDesC& /*aTitle*/, const TDesC& /*aComment*/){};
	virtual void CommentPopDown(){};
	virtual void EditCommentPopUpL(TInt /*aIndex*/, const TDesC& /*aTitle*/, const TDesC& /*aText*/){};
	virtual void ModeHighlightL(){}

public:			// implements MEGnuChessModelTimeObserver
	virtual void UpdateTimeClockL(TBool aWhite, const TDesC& aTimeString, TBool aHighlight);

protected:
	CEGnuChessViewCtl(CEGnuChessModel& aModel);
	void ConstructL();
	void SwitchToStandbyView();

protected:		// from CCoeControlContainer
	virtual void DoLayoutChildren();

	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
	void HandleResourceChange(TInt aType);

	CEGnuChessBoardControl*		iEGnuChessBoardControl;
	CEGnuChessModel&			iModel;
	CBitmappedLabel*			iInfoLabel;
	CBitmappedLabel*			iWhiteTimeLabel;
	CBitmappedLabel*			iBlackTimeLabel;
	TBool						iIsSmall;
	TBuf<64>					iTimeText;
	TInt						iActivateCount;
};


#endif			/* __E_GNU_CHESS_VIEW_H */
