/*******************************************************************
 *
 *	File:		ColorPicker.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "ColorPicker.h"


#include <eikenv.h>
#include <eiklabel.h>
#include "EGnuChessMiscCtls.h"
#include "debug.h"



class CColorChoserColorGraph : public CBitmappedCoeControl
{
public:
	void SetSatAndHue(TInt aHue, TInt aSat);
	void SatAndHue(TInt& aHue, TInt& aSat);

protected:
	CCoeControl* ComponentControl(TInt /*aIndex*/) const{return NULL;}
	TInt CountComponentControls() const {return 0;}
	void DrawGraph(CBitmapContext& aBitmapContext) const;
	void DrawCross(CBitmapContext& aBitmapContext) const;
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	void SetColorPoint(const TPoint& aPoint);

protected:		// from CBitmappedCoeControl
	void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const;
	void Draw(const TRect& aRect) const;

	TPoint	iColorPoint;
	TBool	iNoCross;
	TInt	iHue;
	TInt	iSat;
};

class CColorChoserLumBar : public CBitmappedCoeControl
{
public:
	TInt Lum() {return iLum;}
	void SetSatAndHue(TInt aHue, TInt aSat);
protected:
	CCoeControl* ComponentControl(TInt /*aIndex*/) const{return NULL;}
	TInt CountComponentControls() const {return 0;}
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	void SetLumPoint(const TPoint& aPoint);

	void DrawBar(CBitmapContext& aBitmapContext, const TRect& aRect) const;
	void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const;
	TInt	iHue;
	TInt	iSat;
	TInt	iLum;
};

class CColorChoserLumSlider : public CCoeControl
{
public:
	void SetLum(TInt aLum);
	TInt Lum() {return iLum;}

protected:
	CCoeControl* ComponentControl(TInt /*aIndex*/) const{return NULL;}
	TInt CountComponentControls() const {return 0;}
	void DrawArrow(CBitmapContext& aBitmapContext) const;
	void Draw(const TRect& aRect) const;
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	void SetLumPoint(const TPoint& aPoint);

	TInt		iLum;
	TPoint		iLumPoint;
};


class CColorChoserColorPreview : public CCoeControl
{
public:
	void SetColor(const TRgb& aColor){iColor = aColor;}
	TRgb Color(){return iColor;}

protected:
	CCoeControl* ComponentControl(TInt /*aIndex*/) const{return NULL;}
	TInt CountComponentControls() const {return 0;}
	void DrawBar(CBitmapContext& aBitmapContext) const;
	void Draw(const TRect& aRect) const;
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);

	TRgb	iColor;
};


static void ConfinePointToRect(TPoint& aPoint, const TRect& aRect)
{
	if (aPoint.iX < aRect.iTl.iX)
		aPoint.iX = aRect.iTl.iX;
	else if (aPoint.iX > aRect.iBr.iX)
		aPoint.iX = aRect.iBr.iX;
	if (aPoint.iY < aRect.iTl.iY)
		aPoint.iY = aRect.iTl.iY;
	else if (aPoint.iY > aRect.iBr.iY)
		aPoint.iY = aRect.iBr.iY;

}



/************************************************
 *
 * CC_HSLtoRGB and CC_RGBtoHSL "borrowed" from
 * the WINE project. ;-)
 *
 ************************************************/

#define min(p1,p2) (((p1)<(p2))? (p1) : (p2))
#define max(p1,p2) (((p1)>(p2))? (p1) : (p2))
#if 0
template<class T>
T min(T p1, T p2) { return (p1<p2)? p1 : p2;}

template<class T>
T max(T p1, T p2) { return (p1>p2)? p1 : p2;}
#endif

static int CC_HSLtoRGB(char c, int hue, int sat, int lum)
{
 int res = 0, maxrgb;

 /* hue */
 switch(c)
 {
  case 'R': if (hue > 80)  hue -= 80; else hue += 160; break;
  case 'G': if (hue > 160) hue -= 160; else hue += 80; break;
  case 'B': break;
 }

 /* l below 120 */
 maxrgb = (256 * min(120,lum)) / 120;  /* 0 .. 256 */
 if (hue < 80)
  res = 0;
 else
  if (hue < 120)
  {
   res = (hue - 80) * maxrgb;           /* 0...10240 */
   res /= 40;                        /* 0...256 */
  }
  else
   if (hue < 200)
    res = maxrgb;
   else
    {
     res= (240 - hue) * maxrgb;
     res /= 40;
    }
 res = res - maxrgb / 2;                 /* -128...128 */

 /* saturation */
 res = maxrgb / 2 + (sat * res) / 240;    /* 0..256 */

 /* lum above 120 */
 if (lum > 120 && res < 256)
  res += ((lum - 120) * (256 - res)) / 120;

 return min(res, 255);
}

static int CC_RGBtoHSL(char c, int r, int g, int b)
{
 TInt maxi, mini, mmsum, mmdif, result = 0;
 int iresult = 0;

 maxi = max(r, b);
 maxi = max(maxi, g);
 mini = min(r, b);
 mini = min(mini, g);

 mmsum = maxi + mini;
 mmdif = maxi - mini;

 switch(c)
 {
  /* lum */
  case 'L': mmsum *= 120;              /* 0...61200=(255+255)*120 */
	   result = mmsum / 255;        /* 0...240 */
	   break;
  /* saturation */
  case 'S': if (!mmsum)
	    result = 0;
	   else
	    if (!mini || maxi == 255)
	     result = 240;
	   else
	   {
	    result = mmdif * 240;       /* 0...61200=255*240 */
	    result /= (mmsum > 255 ? mmsum = 510 - mmsum : mmsum); /* 0..255 */
	   }
	   break;
  /* hue */
  case 'H': if (!mmdif)
	    result = 160;
	   else
	   {
	    if (maxi == r)
	    {
	     iresult = 40 * (g - b);       /* -10200 ... 10200 */
	     iresult /= (int) mmdif;    /* -40 .. 40 */
	     if (iresult < 0)
	      iresult += 240;          /* 0..40 and 200..240 */
	    }
	    else
	     if (maxi == g)
	     {
	      iresult = 40 * (b - r);
	      iresult /= (int) mmdif;
	      iresult += 80;           /* 40 .. 120 */
	     }
	     else
	      if (maxi == b)
	      {
	       iresult = 40 * (r - g);
	       iresult /= (int) mmdif;
	       iresult += 160;         /* 120 .. 200 */
	      }
	    result = iresult;
	   }
	   break;
 }
 return result;    /* is this integer arithmetic precise enough ? */
}


/***********************************************************
 *
 * CColorChoserColorGraph
 *
 ***********************************************************/

const TInt KCrossBarWidth = 6;
const TInt KCrossBarHeight = 1;

#define XSTEPS 80
#define YSTEPS 80

void CColorChoserColorGraph::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	switch(aPointerEvent.iType)
	{
		case TPointerEvent::EButton1Down:
			iNoCross = ETrue;
			DrawNow();
			SetColorPoint(aPointerEvent.iPosition);
			break;
		case TPointerEvent::EButton1Up:
			iNoCross = EFalse;
			SetColorPoint(aPointerEvent.iPosition);
			DrawNow();
			ReportEventL(MCoeControlObserver::EEventStateChanged);
			break;
		case TPointerEvent::EDrag:
			iNoCross = EFalse;
			SetColorPoint(aPointerEvent.iPosition);
			ReportEventL(MCoeControlObserver::EEventStateChanged);
			break;
		default:
			break;
	}
}

void CColorChoserColorGraph::DrawGraph(CBitmapContext& aBitmapContext) const
{
	TInt sdif, hdif, xdif, ydif, hue, sat;
	xdif = Size().iWidth / XSTEPS;
	ydif = Size().iHeight / YSTEPS; // +1;
	hdif = 240 / XSTEPS;
	sdif = 240 / YSTEPS;

	aBitmapContext.Reset();
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::ENullPen);
	aBitmapContext.SetClippingRect(Rect());

	TPoint	p(Rect().iTl.iX, Rect().iBr.iY - ydif);
	TSize	size(xdif, ydif);

	for (hue = 0 ; hue < 239 + hdif; hue += hdif)
	{
		for(sat = 0; sat < 240 + sdif; sat += sdif)
		{
			TRect	rect(p, size);
			TInt r = CC_HSLtoRGB('R', hue, sat, 120);
			TInt g = CC_HSLtoRGB('G', hue, sat, 120);
			TInt b = CC_HSLtoRGB('B', hue, sat, 120);

			aBitmapContext.SetBrushColor(TRgb(r, g, b));
			aBitmapContext.DrawRect(rect);

			p.iY-=ydif;
		}
		p.iX+=xdif;
		p.iY=Rect().iBr.iY - ydif;
	}
}

void CColorChoserColorGraph::DrawCross(CBitmapContext& aBitmapContext) const
{
	TSize	horBarSize(KCrossBarWidth, KCrossBarHeight);
	TSize	verBarSize(KCrossBarHeight, KCrossBarWidth);

	aBitmapContext.Reset();
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetBrushColor(KRgbBlack);
	aBitmapContext.SetPenStyle(CGraphicsContext::ENullPen);
	aBitmapContext.SetClippingRect(Rect());

	TPoint	p = iColorPoint + TPoint(0, -1 * (KCrossBarWidth + 1));
	aBitmapContext.DrawRect(TRect(Position() + p, verBarSize));

	p = iColorPoint + TPoint(0, KCrossBarHeight + 1);
	aBitmapContext.DrawRect(TRect(Position() + p, verBarSize));

	p = iColorPoint + TPoint(-1 * (KCrossBarWidth + 1), 0);
	aBitmapContext.DrawRect(TRect(Position() + p, horBarSize));

	p = iColorPoint + TPoint(KCrossBarHeight + 1, 0);
	aBitmapContext.DrawRect(TRect(Position() + p, horBarSize));
}

void CColorChoserColorGraph::DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& /*aBitmapDevice*/) const
{
	DrawGraph(aBitmapContext);
}

void CColorChoserColorGraph::Draw(const TRect& aRect) const
{
	CBitmappedCoeControl::Draw(aRect);
	
	if (!iNoCross)
	{
	    CWindowGc& gc=SystemGc();
		DrawCross(gc);
	}
}


void CColorChoserColorGraph::SetSatAndHue(TInt aHue, TInt aSat)
{
	if ((iHue != aHue) || (aSat != iSat))
	{
		iHue = aHue;
		iSat = aSat;

		TInt x = (Size().iWidth * aHue) / 240;
		TInt y = (Size().iHeight * aSat) / 240;

		iColorPoint = TPoint(x, Size().iHeight - y);
		CBitmappedCoeControl::DrawControl();
	}
}

void CColorChoserColorGraph::SetColorPoint(const TPoint& aPoint)
{
	TSize	size = Size();

	iColorPoint = aPoint - Position();
	ConfinePointToRect(iColorPoint, TRect(Size()));
		
	iHue = (iColorPoint.iX * 240) / size.iWidth;
	iSat = 240 - ((iColorPoint.iY * 240) / size.iHeight);

	ASSERT((iHue>=0 && iHue<=240));
	ASSERT((iSat>=0 && iSat<=240));
}

void CColorChoserColorGraph::SatAndHue(TInt& aHue, TInt& aSat)
{
	aHue = iHue;
	aSat = iSat;
}

/***********************************************************
 *
 * CColorChoserLumBar
 *
 ***********************************************************/

void CColorChoserLumBar::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	switch(aPointerEvent.iType)
	{
		case TPointerEvent::EButton1Down:
		case TPointerEvent::EButton1Up:
		case TPointerEvent::EDrag:
			SetLumPoint(aPointerEvent.iPosition);
			ReportEventL(MCoeControlObserver::EEventStateChanged);
			break;
		default:
			break;
	}
}

void CColorChoserLumBar::SetLumPoint(const TPoint& aPoint)
{
	TSize	size = Size();
	TPoint  lumPoint = aPoint - Position();

	ConfinePointToRect(lumPoint, size);

	iLum = 240 - ((lumPoint.iY * 240) / size.iHeight);
}


void CColorChoserLumBar::DrawBar(CBitmapContext& aBitmapContext, const TRect& /*aRect*/) const
{
	TInt lum, ldif, ydif, r, g, b;

	ldif = 240 / YSTEPS;
	ydif = Size().iHeight / YSTEPS;//+1;

	TPoint	p(Rect().iTl.iX, Rect().iBr.iY - ydif);
	TSize	size(Size().iWidth, ydif);

	aBitmapContext.Reset();
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::ENullPen);
	aBitmapContext.SetClippingRect(Rect());

	for (lum = 0; lum < 240 + ldif; lum += ldif)
	{
		TRect	rect(p, size);

		r = CC_HSLtoRGB('R', iHue, iSat, lum);
		g = CC_HSLtoRGB('G', iHue, iSat, lum);
		b = CC_HSLtoRGB('B', iHue, iSat, lum);

		aBitmapContext.SetBrushColor(TRgb(r, g, b));
		aBitmapContext.DrawRect(rect);

		p.iY-=ydif;
	}
	aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::ESolidPen);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.DrawRect(Rect());
}

void CColorChoserLumBar::DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& /*aBitmapDevice*/) const
{
	DrawBar(aBitmapContext, Rect());
}


void CColorChoserLumBar::SetSatAndHue(TInt aHue, TInt aSat)
{
	if ((iHue != aHue) || (aSat != iSat))
	{
		iHue = aHue;
		iSat = aSat;
		CBitmappedCoeControl::DrawControl();
	}
}

/***********************************************************
 *
 * CColorChoserLumSlider
 *
 ***********************************************************/

const TInt KLumBarSliderArrowHeight = 4;

void CColorChoserLumSlider::DrawArrow(CBitmapContext& aBitmapContext) const
{
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::ESolidPen);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.SetBrushColor(KRgbBlack);
	
		// draw simple arrow
	TPoint	points[3];
	points[0].iX = Position().iX;
	points[0].iY = Position().iY + iLumPoint.iY + KLumBarSliderArrowHeight;
	points[1].iX = Position().iX + Size().iWidth - 1;
	points[1].iY = Position().iY + iLumPoint.iY;
	points[2].iX = points[1].iX;
	points[2].iY = Position().iY + iLumPoint.iY + KLumBarSliderArrowHeight + KLumBarSliderArrowHeight;
	aBitmapContext.DrawPolygon(points, 3);
}

void CColorChoserLumSlider::Draw(const TRect& /*aRect*/) const
{
	CWindowGc& gc = SystemGc();
	gc.SetClippingRect(Rect());
	DrawArrow(gc);
	gc.CancelClippingRect();
}

void CColorChoserLumSlider::SetLum(TInt aLum)
{
	iLum = aLum;
	iLumPoint.iX = 0;
	TInt height = Size().iHeight - (2 * KLumBarSliderArrowHeight);
	TInt y = (height * aLum) / 240;
	iLumPoint.iY = height - y;
}

void CColorChoserLumSlider::SetLumPoint(const TPoint& aPoint)
{
	TSize	size = Size();
	size.iHeight -= (2 * KLumBarSliderArrowHeight);
	iLumPoint = aPoint - Position() - TPoint(0, KLumBarSliderArrowHeight);

	ConfinePointToRect(iLumPoint, size);

	iLum = 240 - ((iLumPoint.iY * 240) / size.iHeight);
}


void CColorChoserLumSlider::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	switch(aPointerEvent.iType)
	{
		case TPointerEvent::EButton1Down:
		case TPointerEvent::EButton1Up:
		case TPointerEvent::EDrag:
			SetLumPoint(aPointerEvent.iPosition);
			ReportEventL(MCoeControlObserver::EEventStateChanged);
			break;
		default:
			break;
	}
}

/***********************************************************
 *
 * CColorChoserColorPreview
 *
 ***********************************************************/

void CColorChoserColorPreview::DrawBar(CBitmapContext& aBitmapContext) const
{
	aBitmapContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::ESolidPen);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.SetBrushColor(iColor);
	aBitmapContext.DrawRect(Rect());
}

void CColorChoserColorPreview::Draw(const TRect& /*aRect*/) const
{
	CWindowGc& gc = SystemGc();
	DrawBar(gc);
}

void CColorChoserColorPreview::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	switch(aPointerEvent.iType)
	{
		case TPointerEvent::EButton1Up:
			if (Rect().Contains(aPointerEvent.iPosition))
				ReportEventL(MCoeControlObserver::EEventStateChanged);
			break;
		default:
			break;
	}
}



/***********************************************************
 *
 * CColorChooserCtl
 *
 ***********************************************************/

void CColorChooserCtl::SizeChanged()
{
	LayoutChildren();
}

void CColorChooserCtl::DoLayoutChildren()
{
		/*
		 * Color graph top left, 160x160
		 */
	TPoint	p = Position();
	iColorChoserColorGraph->SetExtent(p + TPoint(0, KLumBarSliderArrowHeight), TSize(160, 160));

		/*
		 * lum bar next to it with 10 pixel gap, width = 15
		 */
	TRect rect = iColorChoserColorGraph->Rect();
	iColorChoserLumBar->SetExtent(TPoint(rect.iBr.iX + 10, rect.iTl.iY), TSize(15, rect.Size().iHeight));

		/*
		 * lum bar slider strait next to the lum bar
		 */
	rect = iColorChoserLumBar->Rect();
	rect.iTl.iX += rect.Size().iWidth;
	rect.iBr.iX = rect.iTl.iX + 10;
	rect.iTl.iY -= KLumBarSliderArrowHeight;
	rect.iBr.iY += KLumBarSliderArrowHeight;
	iColorChoserLumSlider->SetRect(rect);

		/*
		 * Big color review button, 10 pixels under the color graph
		 */
	rect = iColorChoserColorGraph->Rect();
	rect.iTl.iX = rect.iTl.iX;
	rect.iTl.iY = rect.iBr.iY + 10;
	rect.iBr.iX = rect.iTl.iX + 50;
	rect.iBr.iY = rect.iTl.iY + 20;
	iColorChoserColorPreview->SetRect(rect);

		/*
		 * RGB text, bottom right, 15 pixels high, 200 pixels wide
		 */
	rect = Rect();
	rect.iTl.iX = rect.iTl.iX;
	rect.iTl.iY = rect.iBr.iY - 15;
	rect.iBr.iX = rect.iTl.iX + 200;
	rect.iBr.iY = rect.iBr.iY;
	iLabel->SetRect(rect);

	DoLayoutCustomColorButtons();
}


#define KCustomColorButtonWidth		15
#define KCustomColorButtonHeight	15
#define KCustomColorButtonHorGap	8
#define KCustomColorButtonVerGap	8

void CColorChooserCtl::DoLayoutCustomColorButtons()
{
	TRect rect = iColorChoserLumBar->Rect();

	TPoint topLeftPoint;
	topLeftPoint.iX = rect.iBr.iX - (KCustomColorsHor * KCustomColorButtonWidth) - ((KCustomColorsHor-1) * KCustomColorButtonHorGap);
	topLeftPoint.iY = rect.iBr.iY + KCustomColorButtonVerGap;
	TSize	size(KCustomColorButtonWidth, KCustomColorButtonHeight);

	TPoint p = topLeftPoint;
	for (TInt i=0 ; i<KCustomColorsVer ; i++)
	{
		for (TInt j=0 ; j<KCustomColorsHor ; j++)
		{
			iCustomColors[i*KCustomColorsHor + j]->SetRect(TRect(p, size));
			p.iX += KCustomColorButtonWidth + KCustomColorButtonHorGap;
			iCustomColors[i*KCustomColorsHor + j]->SetColor(KRgbRed);
		}
		p.iX = topLeftPoint.iX;
		p.iY += KCustomColorButtonHeight + KCustomColorButtonVerGap;
	}
}


CColorChooserCtl::CColorChooserCtl()
{
}

CColorChooserCtl::~CColorChooserCtl()
{
	DestroyChildren();
}

void CColorChooserCtl::ConstructL()
{
	CCoeControlContainer::ConstructL();
	Window().PointerFilter(EPointerFilterDrag, 0);

	iColorChoserColorGraph = new(ELeave)CColorChoserColorGraph;
	iColorChoserColorGraph->SetContainerWindowL(*this);
	iColorChoserColorGraph->SetObserver(this);
	AddChildL(iColorChoserColorGraph);	// do not blank child in redraw

	iColorChoserLumBar = new(ELeave)CColorChoserLumBar;
	iColorChoserLumBar->SetContainerWindowL(*this);
	iColorChoserLumBar->SetObserver(this);
	AddChildL(iColorChoserLumBar);	// do not blank child in redraw

	iColorChoserLumSlider = new(ELeave)CColorChoserLumSlider;
	iColorChoserLumSlider->SetContainerWindowL(*this);
	iColorChoserLumSlider->SetObserver(this);
	AddChildL(iColorChoserLumSlider, ETrue);	// blank child in redraw

	iColorChoserColorPreview = new(ELeave)CColorChoserColorPreview;
	iColorChoserColorPreview->SetContainerWindowL(*this);
	iColorChoserColorPreview->SetObserver(this);
	AddChildL(iColorChoserColorPreview);	// do not blank child in redraw

	for (TInt i=0 ; i<KCustomColors ; i++)
	{
		iCustomColors[i] = new(ELeave)CColorChoserColorPreview;
		iCustomColors[i]->SetContainerWindowL(*this);
		iCustomColors[i]->SetObserver(this);
		AddChildL(iCustomColors[i]);	// do not blank child in redraw
	}
	iLabel = new(ELeave)CBitmappedLabel;
	iLabel->SetBgColor(iEikonEnv->Color(EColorDialogBackground));
	iLabel->SetContainerWindowL(*this);
	AddChildL(iLabel, EFalse);	// blank child in redraw

	iFocussedPreviewButton = iColorChoserColorPreview;
}


void CColorChooserCtl::LumChangedL(TInt aLum)
{
	TInt	hue, sat;
	iColorChoserColorGraph->SatAndHue(hue, sat);
	iColorChoserLumSlider->SetLum(aLum);
	DrawChildNow(iColorChoserLumSlider);	// update arrow
	ColorChangedL(hue, sat, aLum);
}

void CColorChooserCtl::HueAndSatChangedL(TInt aHue, TInt aSat)
{
	iColorChoserLumBar->SetSatAndHue(aHue, aSat);
	DrawChildNow(iColorChoserLumBar);
	TInt lum = iColorChoserLumSlider->Lum();
	ColorChangedL(aHue, aSat, lum);
}

void CColorChooserCtl::ColorChangedL(TInt aHue, TInt aSat, TInt aLum)
{
	TInt r = CC_HSLtoRGB('R', aHue, aSat, aLum);
	TInt g = CC_HSLtoRGB('G', aHue, aSat, aLum);
	TInt b = CC_HSLtoRGB('B', aHue, aSat, aLum);

	iColor = TRgb(r, g, b);
	SetColorTextL();		// redraw
	SetPreviewColor();		// redraw
}

void CColorChooserCtl::SetPreviewColor(TBool aRedraw)
{
	iColorChoserColorPreview->SetColor(iColor);
	if (aRedraw)
		DrawChildNow(iColorChoserColorPreview);
	if (iFocussedPreviewButton != iColorChoserColorPreview)
	{
		iFocussedPreviewButton->SetColor(iColor);
		if (aRedraw)
			DrawChildNow(iFocussedPreviewButton);
	}
}


void CColorChooserCtl::SetColor(const TRgb& aColor, TBool aRedraw)
{
	iColor = aColor;

	TInt hue = CC_RGBtoHSL('H', iColor.Red(), iColor.Green(), iColor.Blue());
	TInt sat = CC_RGBtoHSL('S', iColor.Red(), iColor.Green(), iColor.Blue());
	TInt lum = CC_RGBtoHSL('L', iColor.Red(), iColor.Green(), iColor.Blue());

	iColorChoserColorGraph->SetSatAndHue(hue, sat);
	DrawChildNow(iColorChoserColorGraph);

	iColorChoserLumBar->SetSatAndHue(hue, sat);
	DrawChildNow(iColorChoserLumBar);

	iColorChoserLumSlider->SetLum(lum);
	DrawChildNow(iColorChoserLumSlider);
	SetColorTextL(EFalse);
	SetPreviewColor(EFalse);
	if (aRedraw)
		DrawNow();
}

void CColorChooserCtl::SetColorTextL(TBool aRedraw)
{
	TBuf<64>	text;
	text.Format(_L("RGB = (%d, %d, %d)"), iColor.Red(), iColor.Green(), iColor.Blue());
	iLabel->SetTextL(text);
	if (aRedraw)
		DrawChildNow(iLabel);
}

TRgb CColorChooserCtl::Color()
{
	return iColor;
}

void CColorChooserCtl::HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType)
{
	if (aEventType == EEventStateChanged)
	{
		if (aControl == iColorChoserColorGraph)
		{
			TInt hue, sat;
			iColorChoserColorGraph->SatAndHue(hue, sat);
			HueAndSatChangedL(hue, sat);
		}
		else if (aControl == iColorChoserLumSlider)
		{
			LumChangedL(iColorChoserLumSlider->Lum());
		}
		else if (aControl == iColorChoserLumBar)
		{
			LumChangedL(iColorChoserLumBar->Lum());
		}
		else if (aControl == iColorChoserColorPreview)
		{
			FocusToPreviewButton(iColorChoserColorPreview);
		}
		else
		{
			FocusToPreviewButton((CColorChoserColorPreview*)aControl);
		}
	}
}

TRect CColorChooserCtl::FocussedRect(CCoeControl* aControl) const
{
	TRect rect = aControl->Rect();
	rect.Grow(TSize(2, 2));
	return rect;
}

void CColorChooserCtl::FocusToPreviewButton(CColorChoserColorPreview* aControl)
{
	if (iFocussedPreviewButton != aControl)
	{
		Window().Invalidate(FocussedRect(iFocussedPreviewButton));
		
		iFocussedPreviewButton = aControl;
		Window().Invalidate(FocussedRect(iFocussedPreviewButton));
	}
	if (iFocussedPreviewButton != iColorChoserColorPreview)
	{
		SetColor(iFocussedPreviewButton->Color(), ETrue);
	}
}

void CColorChooserCtl::DrawFocussedRect(CBitmapContext& aBitmapContext, const TRect& aRect) const
{
	aBitmapContext.Reset();
	aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::EDashedPen);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.DrawRect(aRect);
}

void CColorChooserCtl::Draw(const TRect& aRect) const
{
	CCoeControlContainer::Draw(aRect);

	CWindowGc& gc = SystemGc();
	DrawFocussedRect(gc, FocussedRect(iFocussedPreviewButton));
}

void CColorChooserCtl::SetCustomColors(const TCustomColors& aCustomColors)
{
	for (TInt i=0 ; i<KCustomColors ; i++)
	{
		if (iCustomColors[i])
			iCustomColors[i]->SetColor(aCustomColors.iColors[i]);
	}
}

void CColorChooserCtl::CustomColors(TCustomColors& aCustomColors)
{
	for (TInt i=0 ; i<KCustomColors ; i++)
	{
		if (iCustomColors[i])
			aCustomColors.iColors[i] = iCustomColors[i]->Color();
	}
}


