/*******************************************************************
 *
 *	File:		EGnuChessAppUi.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikmenup.h>
#include <eikmenub.h>
#include <QuartzKeys.h>

#include <QikSelectMediaFileDlg.h>
#include <QikEditFileFoldersDlg.h>
#include <QikSelectAttachmentTypeDlg.h>
#include <QikSaveFileInFolderDlg.h>

#include "EGnuChessApplication.h"
#include "EGnuChessAppUi.h"
#include "EGnuChessView.h"
#include "EGnuChessUids.h"
#include "EGnuChessDialogs.h"

#include "EGnuChessApp.hrh"
#include <EChess.rsg>
#include "echess.mbg"

_LIT(KCraftyRcFile, "c:\\documents\\echess\\crafty.rc");
_LIT(KDefaultCraftyRcFile, "c:\\documents\\echess\\craftydef.rc");

static void BitmapFileName(TDes& aFilename)
{
	CEikonEnv* env = CEikonEnv::Static();

		/*
		 * The .mbm file is stored in same directory as the .exe
		 * Note that I don't use CEikEnv::CreateBitmapL() which shares bitmaps if loaded 
		 * more than once (it calls CWsBitmap::Load with the sharable flag set).
		 */
	aFilename = ((CEikAppUi*)env->AppUi())->Application()->AppFullName();
	TInt		l = aFilename.Length();
	if (l > 3)
	{
		aFilename[l-1] = 'm';
		aFilename[l-2] = 'b';
		aFilename[l-3] = 'm';
	}
}


TInt SystemActivityGenerator(TAny* /*aPtr*/)
{
	TRawEvent	event;
	event.Set(TRawEvent::EActive);
	UserSvr::AddEvent(event);
	return 0;
}


/******************************************
 *
 * CToolbarIconFlasher
 *
 ******************************************/

CToolbarIconFlasher* CToolbarIconFlasher::NewL()
{
	CToolbarIconFlasher*	self = new(ELeave)CToolbarIconFlasher;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


CToolbarIconFlasher::~CToolbarIconFlasher()
{
	delete iPeriodic;
}


void CToolbarIconFlasher::ConstructL()
{
	iPeriodic = CPeriodic::NewL(CActive::EPriorityHigh);
}

void CToolbarIconFlasher::Start(CEikButtonGroupContainer* aToolbar, TInt aCommand)
{
	ASSERT(iPeriodic);
	iPeriodic->Cancel();
	iToolbar = aToolbar;
	iCommand = aCommand;
	iStep = 0;
	iPeriodic->Start(TTimeIntervalMicroSeconds32(0), TTimeIntervalMicroSeconds32(1000 * 250), TCallBack(PeriodicCallback, this));
}

TInt CToolbarIconFlasher::PeriodicCallback(TAny* aAny)
{
	((CToolbarIconFlasher*) aAny)->DoFlash();
	return 0;
}

void CToolbarIconFlasher::DoFlash()
{
	iToolbar->MakeCommandVisible(iCommand, (iStep & 1));
	iStep++;
	if (iStep == 4)
		iPeriodic->Cancel();
}



CEGnuChessAppUi::~CEGnuChessAppUi()
{
	if (iFOView)
	{
		DeregisterView(*iFOView);
		delete iFOView;
	}
	if (iFCView)
	{
		DeregisterView(*iFCView);
		delete iFCView;
	}

	if (iEGnuChessViewCtl)
	{
		RemoveFromStack(iEGnuChessViewCtl);
		delete iEGnuChessViewCtl;
	}
	delete iToolbarIconFlasher;
}

//#define  __WANT_HEAP_LOGGING__ 1
#undef __WANT_HEAP_LOGGING__ 

#ifdef __WANT_HEAP_LOGGING__
TInt HeapLoggerCallBack(TAny* /*aPtr*/)
{
	TInt cells = User::CountAllocCells();
	TInt mem;
	User::AllocSize(mem);
	RDebug::Print(_L("HEAP: cels=%d, size=%d KB"), cells, mem/1024);
	return 0;
}
#endif


void CEGnuChessAppUi::EnableSelectGame(TBool aEnable)
{
	iToolbar->MakeCommandVisible(EToolBarCmdSelectGame, aEnable);
}


void CEGnuChessAppUi::ConstructL()
{
	BaseConstructL();
	SetToolbarL(R_ECHESS_TOOLBAR);
	iToolbar = iEikonEnv->AppUiFactory()->ToolBar();
	EnableSelectGame(EFalse);

	iToolbarIconFlasher = CToolbarIconFlasher::NewL();

	iCoeEnv->RootWin().SetName(_L("EGNUCHESS_FRONTEND"));

	Model().AddModelGameObserver(*this);

	iEGnuChessViewCtl = CEGnuChessViewCtl::NewL(Model());
	iEGnuChessViewCtl->SetBoardSettings(Model().ChessOptions().iBoardSettings);

	TBool	isFlipOpen = (iCoeEnv->ScreenDevice()->CurrentScreenMode() == 0)  ? ETrue : EFalse;
	if (isFlipOpen)
	{
		// Create and activate the view
		iFOView=new(ELeave) CEGnuChessFOView(Model(), *iEGnuChessViewCtl);
		RegisterViewL(*iFOView);

		iFCView=new(ELeave) CEGnuChessFCView(Model(), *iEGnuChessViewCtl);
		RegisterViewL(*iFCView);
	}
	else
	{
		iFCView=new(ELeave) CEGnuChessFCView(Model(), *iEGnuChessViewCtl);
		RegisterViewL(*iFCView);

		// Create and activate the view
		iFOView=new(ELeave) CEGnuChessFOView(Model(), *iEGnuChessViewCtl);
		RegisterViewL(*iFOView);
	}

	ActivateChessViewL();

	iSystemActivityTimer = CPeriodic::NewL(CActive::EPriorityHigh);
	GamePausedOrResume(EFalse);		// get going
	
#ifdef __WANT_HEAP_LOGGING__
	CPeriodic*	timer = CPeriodic::NewL(CActive::EPriorityHigh);
	timer->Start(TTimeIntervalMicroSeconds32(0), TTimeIntervalMicroSeconds32(1000*1000*2), TCallBack(HeapLoggerCallBack));
#endif
}


void CEGnuChessAppUi::ActivateChessViewL()
{
	TBool	isFlipOpen = (iCoeEnv->ScreenDevice()->CurrentScreenMode() == 0)  ? ETrue : EFalse;

	TVwsViewId	viewId;
	viewId.iAppUid = KUidEGnuChessApp;
	viewId.iViewUid = isFlipOpen ? KUidChessFOView : KUidChessFCView;

	TRAPD(err, ActivateViewL(viewId));
	if (err)
		ActivateTopViewL();
}


void CEGnuChessAppUi::HandleApplicationSpecificEventL(TInt aType, const TWsEvent& aEvent)
{
	if (aType == EEventUser+1)
	{
		ActivateViewL(TVwsViewId(KUidEGnuChessApp, KUidChessFOView));
	}
	else
		CQikAppUi::HandleApplicationSpecificEventL(aType, aEvent);
}


void CEGnuChessAppUi::ClockOptionsL()
{
	// get copy of settings
	TEGnuChessOptions options = Model().ChessOptions();
	CClockSettingsDialog*	dialog = new(ELeave)CClockSettingsDialog(options.iClockSettings);
	if (dialog->ExecuteLD(R_CLOCK_SETTINGS_DIALOG) == EEikBidOk)
	{
		if (options.Save(iEikonEnv->FsSession()))
		{
			Model().SetChessOptions(options);
			Model().SetTimeControl();
			Reset(TRUE, TRUE);
		}
	}
}

void CEGnuChessAppUi::BoardOptionsL()
{
	// get copy of settings
	TEGnuChessOptions options = Model().ChessOptions();
	CBoardSettingsDialog*	dialog = new(ELeave) CBoardSettingsDialog(options.iBoardSettings, options.iCustomColors);
	if (dialog->ExecuteLD(R_BOARD_SETTINGS_DIALOG) == EEikBidOk)
	{
		if (options.Save(iEikonEnv->FsSession()))
		{
			Model().SetChessOptions(options);
			iEGnuChessViewCtl->SetBoardSettings(options.iBoardSettings);
		}
	}
}

void CEGnuChessAppUi::EditTagsL()
{
	CEditTagsDialog::RunLD();
}

void CEGnuChessAppUi::EditCommentsL()
{
	iUserInitiatedComment = ETrue;
	EditCommentEvent();
	iUserInitiatedComment = EFalse;
}

void CEGnuChessAppUi::LoadGameL()
{
	_LIT(KMime, "application/echess");
	_LIT(KDummyAudio, "image/foobar");
	CDesCArray* mimeArray=new( ELeave ) CDesCArrayFlat( 2 );
	CleanupStack::PushL(mimeArray);
	mimeArray->AppendL(KMime);
	mimeArray->AppendL(KDummyAudio);

	TFileName	fileName;
	if (CQikSelectMediaFileDlg::RunDlgLD(*mimeArray, fileName))
	{
		iGameNo = -1;
		TInt noOfGames = -1;
		Model().LoadGame(fileName, noOfGames);
		if (noOfGames > 0)
		{
			SelectGameL();
		}
		EnableSelectGame((noOfGames > 0));
	}

	CleanupStack::PopAndDestroy();		// mimeArray
}


void CEGnuChessAppUi::AboutL()
{
	CEikDialog*		dialog = new CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}

void CEGnuChessAppUi::SaveGameL()
{
	if (Model().iFileName.Length() == 0)
	{
		const TPtrC	ptrDocName = _L("echess.pgn");
		const TPtrC	ptrDocTitle = _L("Save chess game...");
		CSaveDocumentDialog*	dialog = new CSaveDocumentDialog(ptrDocName, ptrDocTitle, Model().iFileName);
		if (dialog->ExecuteLD(R_SAVE_DOCUMENT_DIALOG) == EEikBidOk)
		{
			Model().SaveGame();
		}
	}
	else
	{
		Model().SaveGame();
	}
}


void CEGnuChessAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit:
		{
			ExitEvent(0);
			User::After(500 * 1000);	// wait a bit
			Exit();
		}
			break; 
		
		case ECmdGameSave:
			SaveGameL();
			break;

		case ECmdGameAbout:
			AboutL();
			break;

		case EToolBarCmdSelectGame:
			SelectGameL();
			break;

		case ECmdModeMachineBlack:
			MachineBlackEvent();
			DrawPosition(TRUE, NULL);
			break;

		case ECmdModeMachineWhite:
			MachineWhiteEvent();
			DrawPosition(TRUE, NULL);
			break;

		case ECmdOptionsFlipView:
			flipView = !flipView;
			DrawPosition(TRUE, NULL);
			break;

		case ECmdModePause:
		case EToolBarCmdPause:
			PauseEvent();
		  break;

		case ECmdStepBackward:
		case EToolBarCmdStepBackward:
			BackwardEvent();
			break;

		case ECmdStepForward:
		case EToolBarCmdStepForward:
			ForwardEvent();
			break;

		case ECmdStepBackToStart:
		case EToolBarCmdStepBackToStart:
			ToStartEvent();
			break;

		case ECmdStepForwardToEnd:
		case EToolBarCmdStepForwardToEnd:
			ToEndEvent();
			break;

		case ECmdStepRevert:
			RevertEvent();
			break;

		case ECmdStepTruncateGame:
			TruncateGameEvent();
			break;

		case ECmdStepMoveNow:
			MoveNowEvent();
			break;

		case ECmdStepRetractMove:
			RetractMoveEvent();
			break;

		case ECmdGameReset:
			ResetUI();
			ResetGameEvent();
			break;

		case ECmdGameLoad:
			LoadGameL();
			break;

		case ECmdOptionsBoard:
			BoardOptionsL();
			break;

		case EToolBarCmdEditTags:
			EditTagsL();
			break;

		case EToolBarCmdEditComments:
			EditCommentsL();
			break;

		case ECmdModeEditGame:
			EditGameEvent();
			break;

		case ECmdActionAccept:
			AcceptEvent();
			break;

		case ECmdActionDecline:
			DeclineEvent();
			break;

		case ECmdActionRematch:
			RematchEvent();
			break;

		case ECmdActionCallFlag:
			CallFlagEvent();
			break;

		case ECmdActionDraw:
			DrawEvent();
			break;

		case ECmdActionAdjourn:
			AdjournEvent();
			break;

		case ECmdActionAbort:
			AbortEvent();
			break;

		case ECmdActionResign:
			ResignEvent();
			break;


		case ECmdOptionsClock:
			ClockOptionsL();
			break;

		case ECmdActionSendCommand:
			SendCommandL();
			break;

		case ECmdActionEditCraftyRc:
			EditCraftyRcL();
			break;

		default:
			break;
	}
}

void CEGnuChessAppUi::DynInitFileMenuPaneL(CEikMenuPane* /*aMenuPane*/)
{
}

void CEGnuChessAppUi::DynInitModeMenuPaneL(CEikMenuPane* aMenuPane)
{
	UpdateModeMenuPaneL(aMenuPane);
	UpdatePauseMenuPaneL(aMenuPane);
}

void CEGnuChessAppUi::UpdatePauseMenuPaneL(CEikMenuPane* aMenuPane)
{
	_LIT(KMenuItemPause, "Pause ");
	_LIT(KMenuItemResume, "Resume");
	aMenuPane->SetItemTextL(ECmdModePause, pausing ? KMenuItemResume : KMenuItemPause);
}

void CEGnuChessAppUi::UpdateModeMenuPaneL(CEikMenuPane* aMenuPane)
{
	TInt nowModeChecked = iModeNowChecked;

	switch(gameMode)
	{
		case BeginningOfGame:
			if (appData.icsActive)
				nowModeChecked = 0;
			else if (appData.noChessProgram)
				nowModeChecked = ECmdModeEditGame;
			else
				nowModeChecked = ECmdModeMachineBlack;
			break;
		case MachinePlaysBlack:
			nowModeChecked = ECmdModeMachineBlack;
			break;
		case MachinePlaysWhite:
			nowModeChecked = ECmdModeMachineWhite;
			break;
		case TwoMachinesPlay:
			nowModeChecked = 0;
			break;
		case AnalyzeMode:
			nowModeChecked = 0;
			break;
		case AnalyzeFile:
			nowModeChecked = 0;
			break;
		case EditGame:
			nowModeChecked = ECmdModeEditGame;
			break;
		case PlayFromGameFile:
			nowModeChecked = 0;
			break;
		case EditPosition:
			nowModeChecked = ECmdModeEditPosition;
			break;
		case Training:
			nowModeChecked = 0;
			break;
		case IcsPlayingWhite:
		case IcsPlayingBlack:
		case IcsObserving:
		case IcsIdle:
			nowModeChecked = 0;
			break;
		default:
		case EndOfGame:
			nowModeChecked = 0;
			break;
	}

	if (iModeNowChecked && (iModeNowChecked != nowModeChecked))
	{
		aMenuPane->ItemData(iModeNowChecked).iFlags &= !EEikMenuItemCheckBox;
		aMenuPane->SetItemButtonState(iModeNowChecked, EEikMenuItemSymbolOn);
	}
	if (nowModeChecked && (iModeNowChecked != nowModeChecked))
	{
		aMenuPane->ItemData(nowModeChecked).iFlags |= EEikMenuItemCheckBox;
		aMenuPane->SetItemButtonState(nowModeChecked, EEikMenuItemSymbolOn);
	}
	nowModeChecked = iModeNowChecked;
}


void CEGnuChessAppUi::DynInitActionMenuPaneL(CEikMenuPane* /*aMenuPane*/)
{
//	aMenuPane->DeleteMenuItem(ECmdActionSendCommand);
}

void CEGnuChessAppUi::DynInitStepMenuPaneL(CEikMenuPane* /*aMenuPane*/)
{
}

void CEGnuChessAppUi::DynInitOptionsMenuPaneL(CEikMenuPane* aMenuPane)
{
	CEikMenuPaneItem::SData&	flipMenuItem = aMenuPane->ItemData(ECmdOptionsFlipView);
	if (flipView)
		flipMenuItem.iFlags |= EEikMenuItemCheckBox;
	else
		flipMenuItem.iFlags &= ~EEikMenuItemCheckBox;
	aMenuPane->SetItemButtonState(ECmdOptionsFlipView, EEikMenuItemSymbolOn);
}


void CEGnuChessAppUi::DynInitFcMenuPaneL(CEikMenuPane* aMenuPane)
{
	UpdatePauseMenuPaneL(aMenuPane);
}

void CEGnuChessAppUi::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
{
	switch (aMenuId)
	{
		case R_GAME_MENU:
			DynInitFileMenuPaneL(aMenuPane);
			break;
		case R_MODE_MENU:
			DynInitModeMenuPaneL(aMenuPane);
			break;
		case R_ACTIONS_MENU:
			DynInitActionMenuPaneL(aMenuPane);
			break;
		case R_STEP_MENU:
			DynInitStepMenuPaneL(aMenuPane);
			break;
		case R_OPTIONS_MENU:
			DynInitOptionsMenuPaneL(aMenuPane);
			break;
		case R_FC_MENU:
			DynInitFcMenuPaneL(aMenuPane);
			break;
		default:
			break;
	}
}

void CEGnuChessAppUi::UserThinksL()
{
}

void CEGnuChessAppUi::MachineThinksL()
{
}


void CEGnuChessAppUi::GamePausedOrResume(TBool aIsPaused)
{
	iSystemActivityTimer->Cancel();
	if (aIsPaused)
	{
		SendToProgram("ponder off\n", &first);
	}
	else
	{
		iSystemActivityTimer->Start(TTimeIntervalMicroSeconds32(0), TTimeIntervalMicroSeconds32(1000*1000*5), TCallBack(SystemActivityGenerator));
		SendToProgram("ponder on\n", &first);
	}
}


void CEGnuChessAppUi::ModeHighlightL()
{
	if (pausing != iPrevPausing)
	{
		TFileName	file;
		BitmapFileName(file);

		CEikCommandButton* but = (CEikCommandButton*) iToolbar->ControlOrNull(EToolBarCmdPause);
		if (but)
		{
			TInt imageId  = pausing ? EMbmEchessToolbararrowresume : EMbmEchessToolbararrowpause;
			TInt maskId  = pausing ? EMbmEchessToolbararrowresumemask : EMbmEchessToolbararrowpausemask;
			but->SetPictureFromFileL(file, imageId, maskId);
			but->DrawNow();
		}
		GamePausedOrResume(pausing);
		iPrevPausing = pausing;
	}
}


void CEGnuChessAppUi::CommentPopUpL(const TDesC& aTitle, const TDesC& aComment)
{
	if (iUserInitiatedComment)
	{
		iUserInitiatedComment = EFalse;
		CEditCommentsDialog::RunLD(-1, aTitle, aComment);
	}
	else
		iToolbarIconFlasher->Start(iToolbar, EToolBarCmdEditComments);
}

void CEGnuChessAppUi::CommentPopDown()
{
}


void CEGnuChessAppUi::EditCommentPopUpL(TInt aIndex, const TDesC& aTitle, const TDesC& aComment)
{
	if (iUserInitiatedComment)
	{
		iUserInitiatedComment = EFalse;
		CEditCommentsDialog::RunLD(aIndex, aTitle, aComment);
	}
	else
		iToolbarIconFlasher->Start(iToolbar, EToolBarCmdEditComments);
}

void CEGnuChessAppUi::SelectGameL()
{
	TParsePtrC	parse(Model().iFileName);
	if (CSelectGameDialog::RunLD(iGameNo, parse.NameAndExt()) == EEikBidOk)
		Model().SelectedGame(iGameNo, ETrue);
}


void CEGnuChessAppUi::ResetUI()
{
	iGameNo = -1;
	EnableSelectGame(EFalse);
}

void CEGnuChessAppUi::SendCommandL()
{
	TBuf<128>	line;
	CEikDialog*	dialog = new COneLineEdit(line);
	dialog->PrepareLC(R_EDIT_DIALOG);
	dialog->SetTitleL(_L("Send command to engine"));
	if (dialog->RunLD() == EEikBidOk)
	{
		TBuf8<128>	line8;
		line8.Copy(line);
		line8.Append('\n');
		char*	cmd = (char *)line8.PtrZ();
		SendToProgram(cmd, &first);
	}
}

void CEGnuChessAppUi::EditCraftyRcL()
{
	CEditTextFileDialog::RunLD(KCraftyRcFile, KDefaultCraftyRcFile, R_EDIT_FILE_DLG_TITLE);
}


