/*******************************************************************
 *
 *	File:		EGnuChessDialogs.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikenv.h>
#include <eikappui.h>
#include <eikapp.h>
#include <eikedwin.h>
#include <eikchlst.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eikchlst.h>
#include <eiklabel.h>
#include <eikimage.h>

#include <QikEditFileFoldersDlg.h>

#include "EGnuChessDialogs.h"
#include "EGnuChessApp.hrh"
#include <EChess.rsg>
#include <EChess.mbg>
#include "EGnuChessMiscCtls.h"
#include "EGnuChessBoardCtl.h"
#include "ColorPicker.h"
#include "version.h"
#include <stdlib.h>



/**********************************************
 *
 * Save document
 *
 **********************************************/

CSaveDocumentDialog::CSaveDocumentDialog(const TDesC& aDocName, const TDesC& aDocTitle, TFileName& aFileName) :
	iDocTitle(aDocTitle), iFileName(aFileName)
{
	iDocName = aDocName;
}

CSaveDocumentDialog::~CSaveDocumentDialog()
{
	delete iQikMediaFileFolderUtils;
	delete iEChessMimeArray;
}


TBool CSaveDocumentDialog::OkToExitL(TInt aButtonId)
{
	TBool	okToExit = ETrue;

	switch (aButtonId)
	{
		case ECmdSaveDocumentEditFolders:
			EditFolders();
			UpdateFolders();
			okToExit = EFalse;
			break;

		case EEikBidOk:
			if (!GetFileName())
			{
				User::InfoPrint(_L("Document renamed, already existed"));
				okToExit = EFalse;
				iNameEdwin->SetTextL(&iDocName);
				iNameEdwin->DrawNow();
			}
		default:
			break;
	}
	return okToExit;
}

void CSaveDocumentDialog::PreLayoutDynInitL()
{
	iLocationChoiceList	= (CEikChoiceList*) Control(EIdSaveAsLocationChoiceList);
	iFoldersChoiceList	= (CEikChoiceList*) Control(EIdSaveAsFoldersChoiceList);
	iNameEdwin			= (CEikEdwin*) Control(EIdSaveAsDoccumentNameEdwin);

	_LIT(KEChessMime, "application/echess");
	iEChessMimeArray = new(ELeave) CDesCArrayFlat(2);
	iEChessMimeArray->AppendL(KEChessMime);

	iQikMediaFileFolderUtils = CQikMediaFileFolderUtils::NewL(*iEikonEnv);

	UpdateFolders();

	if (iHasExternalDir)
		DeleteLine(EIdSaveAsInternalOnlyLabel);
	else
		DeleteLine(EIdSaveAsLocationChoiceList);
}


void CSaveDocumentDialog::PostLayoutDynInitL()
{
	SetTitleL(iDocTitle);
	iNameEdwin->SetTextL(&iDocName);

	if (iHasExternalDir)
		UpdateLocation();
}

void CSaveDocumentDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case EIdSaveAsLocationChoiceList:
			if (iLocationChoiceList->CurrentItem() == 0)
				iShowExternal = EFalse;
			else
				iShowExternal = ETrue;
			UpdateFolders();
			iLocationChoiceList->SetCurrentItem(0);
			iLocationChoiceList->DrawNow();
			break;
		default:
			break;
	}
}


TBool CSaveDocumentDialog::EditFolders()
{
	CQikMediaFolderMonitor* monitor = CQikMediaFolderMonitor::NewL(*this, iEikonEnv->FsSession(), *iEChessMimeArray, *iEikonEnv);
	CleanupStack::PushL(monitor);
	
	TBool	updateNeeded = EFalse;
	CQikMediaFile* folderInfo= NULL;
	CQikEditFileFoldersDlg::RunDlgLD(*iEChessMimeArray, monitor->iCategoriesList, 0, folderInfo, updateNeeded);

	CleanupStack::PopAndDestroy();		// monitor
	return updateNeeded;
}


void CSaveDocumentDialog::UpdateLocation()
{
	iLocationChoiceList->SetArrayL(R_LOCATION_ARRAY);
}


void CSaveDocumentDialog::UpdateFolders()
{
	iHasExternalDir = EFalse;
	iExternalDrive = EDriveC;
	
	CArrayPtrFlat<CQikMediaFile>* foldersList = new(ELeave) CArrayPtrFlat<CQikMediaFile>(5);
	CleanupStack::PushL(foldersList);
	iQikMediaFileFolderUtils->FoldersL(*iEChessMimeArray, *foldersList);

	CDesCArray*		list = new(ELeave) CDesCArrayFlat(5);
	CleanupStack::PushL(list);
	if (foldersList && foldersList->Count())
	{
		TInt	count = foldersList->Count();
		for (TInt i=0 ; i<count ; i++)
		{
			TDriveNumber	driveNumber = foldersList->At(i)->Drive();
			if (!iHasExternalDir && (driveNumber != EDriveC))
			{
				iHasExternalDir = ETrue;
				iExternalDrive = driveNumber;
			}

			if (iShowExternal)
			{
				if (driveNumber != EDriveC)
					list->AppendL(foldersList->At(i)->Folder());
			}
			else if (driveNumber == EDriveC)
				list->AppendL(foldersList->At(i)->Folder());
		}
	}
	iFoldersChoiceList->SetArrayL(list);
	CleanupStack::Pop(list);
	CleanupStack::PopAndDestroy(foldersList);
}


const TInt KMaxUniqueNum = 99;

TBool CSaveDocumentDialog::GetFileName()
{
	iNameEdwin->GetText(iDocName);

	TBool		unique = EFalse;
	TInt		item = iFoldersChoiceList->CurrentItem();
	TInt		num = 0;
	TFileName	doc = iDocName;

	do
	{
		CQikMediaFile* file = CQikMediaFile::NewL(
					iShowExternal ? iExternalDrive : EDriveC, 
					_L("application"), 
					iFoldersChoiceList->Array()->MdcaPoint(item),
					doc,
					1024);
		CleanupStack::PushL(file);
		iFileName = iQikMediaFileFolderUtils->CreateFileNameL(*file);
		CleanupStack::PopAndDestroy();

		TUint	dummy;
		TInt err = iEikonEnv->FsSession().Att(iFileName, dummy);
		unique =  (err == KErrNotFound);
		if (!unique)
		{
			num++;
			MakeUniqueName(iDocName, doc, num);
		}
	} while (!unique && num < KMaxUniqueNum);

	if (num < KMaxUniqueNum)
		iDocName = doc;

	return (num == 0);
}

void CSaveDocumentDialog::MakeUniqueName(const TDesC& aDocName, TDes& aNumberedDocName, TInt aNum)
{
	TParsePtrC	parse(aDocName);
	TPtrC		name = parse.Name();
	TPtrC		ext = parse.Ext();
	aNumberedDocName.Format(_L("%S-%d%S"), &name, aNum, &ext);
}




/**********************************************
 *
 * Choose color dialog
 *
 **********************************************/



void DumpControls(CCoeControl* aControl, TInt aLevel = 0)
{
	TSize	size = aControl ? aControl->Size() : TSize(-1, -1);
	TBuf<64>	text;
	for(TInt i=0 ; i<aLevel ; i++)
		text.Append(_L("  "));
	text.AppendFormat(_L("(%d) 0x%x, (%d, %d)"), aLevel, aControl, size.iWidth, size.iHeight);
	RDebug::Print(text);
	if (aControl)
	{
		for (TInt i=0 ; i<aControl->CountComponentControls() ; i++)
			DumpControls(aControl->ComponentControl(i), aLevel+1);
	}
}

void DumpChildren(TUint aId)
{
	CCoeControl*	control = (CCoeControl*) aId;
	if (control)
	{
		DumpControls(control);
		DumpChildren(control->DrawableWindow()->NextSibling());
	}
}


CCoeControl* CoeChild(CCoeControl* aControl, TInt aChild)
{
	CCoeControl* child = NULL;
	if (aControl)
	{
		if (aChild<aControl->CountComponentControls())
			child = aControl->ComponentControl(aChild);
	}
	return child;
}

CChooseColorDialog::CChooseColorDialog(TRgb& aColor, TCustomColors& aCustomColors) : 
	iColor(aColor), iCustomColors(aCustomColors)
{
}

TBool CChooseColorDialog::OkToExitL(TInt aButtonId)
{
	if (aButtonId == EEikBidOk)
	{
		// get selected color and (changed) custom colors
		CColorChooserCtl* control = (CColorChooserCtl*)Control(EIdChooseColorCtl);
		iColor = control->Color();
		control->CustomColors(iCustomColors);
	}
	return ETrue;
}

void CChooseColorDialog::PreLayoutDynInitL()
{
	CColorChooserCtl* control = (CColorChooserCtl*)Control(EIdChooseColorCtl);
	control->ConstructL();
	control->SetSize(TSize(100, 256));
}

void CChooseColorDialog::PostLayoutDynInitL()
{
	CColorChooserCtl* control = (CColorChooserCtl*)Control(EIdChooseColorCtl);
	control->SetColor(iColor);
	control->SetCustomColors(iCustomColors);

//	DumpControls(this);

		/*
		 * Hmm, there is a bug in the dialog code that makes a scrollbar appear
		 * when none is needed. After some playing around the following snippet 
		 * seems to cure it.
		 */
	CCoeControl*		stupid_control = CoeChild(CoeChild(CoeChild(this, 1), 1), 0);
	stupid_control->SetSize(stupid_control->Size());
}


void CChooseColorDialog::SetSizeAndPosition(const TSize& aSize)
{
	TSize	size = aSize;
	size.iHeight = 320;
	SetCornerAndSize(EHCenterVCenter, size);
}

SEikControlInfo CChooseColorDialog::CreateCustomControlL(TInt aControlType)
{
	SEikControlInfo controlInfo;

	controlInfo.iControl = NULL;
	controlInfo.iTrailerTextId = 0;
	controlInfo.iFlags = 0;

	if(aControlType == EChooseColorCtlType)
	{
		CCoeControl* control = new(ELeave)CColorChooserCtl;
		controlInfo.iControl = control;
	}

	return(controlInfo);
}

/**********************************************
 *
 * Board options
 *
 **********************************************/


CBoardSettingsDialog::CBoardSettingsDialog(TBoardSettings& aBoardSettings, TCustomColors& aCustomColors)
	: iBoardSettings(aBoardSettings), iCustomColors(aCustomColors)
{
}

CBoardSettingsDialog::~CBoardSettingsDialog()
{
	iChessBitmaps.DestroyAllBitmaps();
}

void CBoardSettingsDialog::ChangeColorL(TRgb& aColor)
{
	TCustomColors	customColors = iCustomColors;
	TRgb			color = aColor;

	CChooseColorDialog*	dialog = new(ELeave) CChooseColorDialog(color, customColors);
	if (dialog->ExecuteLD(R_COLOR_CHOOSER_DIALOG) == EEikBidOk)
	{
		iCustomColors = customColors;
		aColor = color;
		UpdateControls();
	}
}

void CBoardSettingsDialog::ChangePatternL()
{
	TInt pattern = iBoardSettings.iPattern;
	CPatternDialog*	dialog = new(ELeave) CPatternDialog(pattern, iBoardSettings);
	if (dialog->ExecuteLD(R_PATTERN_CHOOSER_DIALOG) == EEikBidOk)
	{
		iBoardSettings.iPattern = pattern;
		iChessBitmaps.ReplacePatterns(iBoardSettings.iPattern);
		UpdateControls();
	}
}


void CBoardSettingsDialog::UpdateControls(TBool aRedraw)
{
	CEGnuChessBoardSingleSquare* control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorWhitePiece);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorBlackPiece);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorLightSquare);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorDarkSquare);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorHighLight);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsPattern);
	control->SetChessBitmaps(iChessBitmaps);
	control->SetBoardSettings(iBoardSettings);
	if (aRedraw)
		control->DrawNow();
}

TBool CBoardSettingsDialog::OkToExitL(TInt aButtonId)
{
	TBool	okToExit = ETrue;

	switch (aButtonId)
	{
		case ECmdDefaults:
			iBoardSettings.Defaults();
			iChessBitmaps.ReplacePatterns(iBoardSettings.iPattern);
			UpdateControls(ETrue);
			okToExit = EFalse;
			break;
		default:
			break;
	}

	return okToExit;
}


void CBoardSettingsDialog::PreLayoutDynInitL()
{
	iChessBitmaps.LoadAllBitmapsL();
	iChessBitmaps.ReplacePatterns(iBoardSettings.iPattern);

	CEGnuChessBoardSingleSquare* control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorWhitePiece);
	control->ConstructL(WhitePawn, ETrue, EFalse);

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorBlackPiece);
	control->ConstructL(BlackKnight, EFalse, EFalse);

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorLightSquare);
	control->ConstructL(BlackBishop, ETrue, EFalse);

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorDarkSquare);
	control->ConstructL(WhiteRook, EFalse, EFalse);

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsColorHighLight);
	control->ConstructL(WhiteQueen, ETrue, ETrue);

	control = (CEGnuChessBoardSingleSquare*) Control(EIdBoardSettingsPattern);
	control->ConstructL(EmptySquare, ETrue, ETrue);
}


void CBoardSettingsDialog::PostLayoutDynInitL()
{
	UpdateControls();
}

SEikControlInfo CBoardSettingsDialog::CreateCustomControlL(TInt aControlType)
{
	SEikControlInfo controlInfo;

	controlInfo.iControl = NULL;
	controlInfo.iTrailerTextId = 0;
	controlInfo.iFlags = 0;

	if(aControlType == ESingleSquareCtlType)
	{
		CCoeControl* control = new(ELeave)CEGnuChessBoardSingleSquare;
		controlInfo.iControl = control;
	}

	return(controlInfo);
}


void CBoardSettingsDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case EIdBoardSettingsColorWhitePiece:
			ChangeColorL(iBoardSettings.iColorWhitePiece);
			break;

		case EIdBoardSettingsColorBlackPiece:
			ChangeColorL(iBoardSettings.iColorBlackPiece);
			break;

		case EIdBoardSettingsColorLightSquare:
			ChangeColorL(iBoardSettings.iColorLightSquare);
			break;

		case EIdBoardSettingsColorDarkSquare:
			ChangeColorL(iBoardSettings.iColorDarkSquare);
			break;

		case EIdBoardSettingsColorHighLight:
			ChangeColorL(iBoardSettings.iColorHighlight);
			break;

		case EIdBoardSettingsPattern:
			ChangePatternL();
			break;

		default:
			break;
	}
}


/**********************************************
 *
 * Choose pattern dialog
 *
 **********************************************/


CPatternDialog::CPatternDialog(TInt& aPattern, const TBoardSettings& aBoardSettings) 
	: iBoardSettings(aBoardSettings), iPattern(aPattern)
{
}

TBool CPatternDialog::OkToExitL(TInt aButtonId)
{
	if (aButtonId == EEikBidOk)
	{
		// get selected color and (changed) custom colors
		CPatternChooserCtl* control = (CPatternChooserCtl*)Control(EIdChooseColorCtl);
		iPattern = control->Pattern();
	}
	return ETrue;
}

void CPatternDialog::PreLayoutDynInitL()
{
	CPatternChooserCtl* control = (CPatternChooserCtl*)Control(EIdChoosePatternCtl);
	control->ConstructL(iBoardSettings);
	control->SetSize(TSize(100, 200));
}

void CPatternDialog::PostLayoutDynInitL()
{
	CPatternChooserCtl* control = (CPatternChooserCtl*)Control(EIdChoosePatternCtl);
	control->SetPattern(iPattern);
}

SEikControlInfo CPatternDialog::CreateCustomControlL(TInt aControlType)
{
	SEikControlInfo controlInfo;

	controlInfo.iControl = NULL;
	controlInfo.iTrailerTextId = 0;
	controlInfo.iFlags = 0;

	if(aControlType == EChoosePatternCtlType)
	{
		CCoeControl* control = new(ELeave)CPatternChooserCtl;
		controlInfo.iControl = control;
	}

	return(controlInfo);
}


/**********************************************
 *
 * Generic edit dialog
 *
 **********************************************/


CEditDialog::CEditDialog()
{
}

void CEditDialog::Text8ToEdwin(const TText8* aText8, TBool aUpdateEdwin)
{
	iText.SetLength(0);
	while (*aText8)
	{
		if (*aText8 == '\n')
		{
			iText.Append(0x2029);
			aText8++;
		}
		else
			iText.Append(*aText8++);
	}
	if (aUpdateEdwin)
	{
		CEikEdwin* edwin = (CEikEdwin*) Control(EIdEditEdwin);
		edwin->SetTextL(&iText);
		edwin->DrawNow();
	}
}

void CEditDialog::Text8FromEdwin()
{
	CEikEdwin* edwin = (CEikEdwin*) Control(EIdEditEdwin);

	edwin->GetText(iText);
	TInt len = iText.Length();
	iText8[0] = 0;
	TInt i=0;

	for (i=0 ; i<len ; i++)
	{
		TText16 c = iText[i];
		if (c == 0x2029)
			iText8[i] = '\n';
		else
			iText8[i] = (TText8) c;
	}
	iText8[i] = 0;
}

void CEditDialog::PostLayoutDynInitL()
{
	CEikEdwin* edwin = (CEikEdwin*) Control(EIdEditEdwin);
	edwin->SetTextL(&iText);
}


/**********************************************
 *
 * Edit tags dialog
 *
 **********************************************/

void CEditTagsDialog::RunLD()
{
	CEikDialog*	dialog = new(ELeave)CEditTagsDialog;
	dialog->PrepareLC(R_EDIT_DIALOG);
	dialog->SetTitleL(R_EDIT_TAGS_DLG_TITLE);
	dialog->RunLD();
}

CEditTagsDialog::CEditTagsDialog()
{
	SetTagsToEdwin();
}

void CEditTagsDialog::SetTagsToEdwin()
{
	char* tagsdata = PGNTags(&gameInfo);
	Text8ToEdwin((const TText8*) tagsdata);
	free(tagsdata);
}


TBool CEditTagsDialog::SaveTags()
{
	Text8FromEdwin();
	TInt err = ReplaceTags(iText8, &gameInfo);
	if (err)
		User::InfoPrint(_L("Tags could not be saved"));

	return (err == 0);
}

TBool CEditTagsDialog::OkToExitL(TInt aButtonId)
{
	TBool okToExit = ETrue;

	if (aButtonId == EEikBidOk)
		okToExit = SaveTags();

	return okToExit;
}


/**********************************************
 *
 * Edit comments dialog
 *
 **********************************************/

void CEditCommentsDialog::RunLD(TInt aIndex, const TDesC& aTitle, const TDesC& aComment)
{
	CEikDialog*	dialog = new(ELeave)CEditCommentsDialog(aIndex, aComment);
	dialog->PrepareLC(R_EDIT_DIALOG);
	dialog->SetTitleL(aTitle);
	dialog->RunLD();
}

CEditCommentsDialog::CEditCommentsDialog(TInt aIndex, const TDesC& aComment) : iIndex(aIndex)
{
	iText = aComment;
}


void CEditCommentsDialog::SaveComments()
{
	Text8FromEdwin();
	ReplaceComment(iIndex, iText8);
}

TBool CEditCommentsDialog::OkToExitL(TInt aButtonId)
{
	if (aButtonId == EEikBidOk)
		SaveComments();

	return ETrue;
}


/**********************************************
 *
 * Select game dialog
 *
 **********************************************/

TInt CSelectGameDialog::RunLD(TInt& aGameNo, const TDesC& aTitle)
{
	CEikDialog*	dialog = new(ELeave)CSelectGameDialog(aGameNo);
	dialog->PrepareLC(R_SELECT_GAME_DIALOG);
	dialog->SetTitleL(aTitle);
	return dialog->RunLD();
}

CSelectGameDialog::CSelectGameDialog(TInt& aGameNo) : iGameNo(aGameNo)
{
}

void CSelectGameDialog::PreLayoutDynInitL()
{
	TBuf<128>	gameName;

	CEikTextListBox* listbox=(CEikTextListBox*)Control(EIdListBoxCtl);
	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());

	ListGame*	lg = (ListGame *) ::gameList.head;
	for (TInt nItem = 0; nItem < ((ListGame *) ::gameList.tailPred)->number; nItem++)
	{
		char *st = ::GameListLine(lg->number, &lg->gameInfo);

		TPtrC8 gameName8((TUint8*)st);
		gameName.Copy(gameName8);
		listboxArray->AppendL(gameName);

		free(st);
		lg = (ListGame *) lg->node.succ;
	}

	if (iGameNo != -1)
		listbox->SetCurrentItemIndex(iGameNo);

	CEikScrollBarFrame* scrollbarFrame = listbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);
}

TBool CSelectGameDialog::OkToExitL(TInt aButtonId)
{
	if (aButtonId == EEikBidOk)
	{
		CEikTextListBox* listbox=(CEikTextListBox*)Control(EIdListBoxCtl);
		iGameNo = listbox->CurrentItemIndex();
	}

	return ETrue;
}


/**********************************************
 *
 * Clock settings dialog
 *
 **********************************************/

CClockSettingsDialog::CClockSettingsDialog(TClockSettings& aClockSettings) :
	iClockSettings(aClockSettings)
{
}

void CClockSettingsDialog::SetEdwinInt(TInt aInt, TInt aEdwinRes)
{
	CEikEdwin*	edwin = (CEikEdwin*) Control(aEdwinRes);
	TBuf<32>	text;
	text.AppendNum(aInt);
	edwin->SetTextL(&text);
}

TBool CClockSettingsDialog::GetEdwinInt(TInt& aInt, TInt aEdwinRes)
{
	TBool	success = EFalse;
	CEikEdwin*	edwin = (CEikEdwin*) Control(aEdwinRes);
	if (edwin)
	{
		TBuf<32>	text;
		edwin->GetText(text);
		TLex		lex(text);
		success = (lex.Val(aInt) == KErrNone);
	}
	return success;
}


void CClockSettingsDialog::UpdateControlsL(TBool aRedraw)
{
	TInt label1Id = (iClockSettings.iClockType == TClockSettings::EClockTypeConventional) 
		? R_CLOCK_SETTINGS_CONV_LABEL1 : R_CLOCK_SETTINGS_INCR_LABEL1;
	TInt label2Id = (iClockSettings.iClockType == TClockSettings::EClockTypeConventional) 
		? R_CLOCK_SETTINGS_CONV_LABEL2 : R_CLOCK_SETTINGS_INCR_LABEL2;

	SetControlCaptionL(EIdClockSettingsEdwin1, label1Id);
	SetControlCaptionL(EIdClockSettingsEdwin2, label2Id);

	if (iClockSettings.iClockType == TClockSettings::EClockTypeConventional)
	{
		SetEdwinInt(iClockSettings.iConventionalNoOfMoves, EIdClockSettingsEdwin1);
		CEikEdwin*	edwin = (CEikEdwin*) Control(EIdClockSettingsEdwin2);
		edwin->SetTextL(&iClockSettings.iConventionalTime);
	}
	else
	{
		CEikEdwin*	edwin = (CEikEdwin*) Control(EIdClockSettingsEdwin1);
		edwin->SetTextL(&iClockSettings.iIncrementalInitialTime);
		SetEdwinInt(iClockSettings.iIncrementalAdditionalTimePerMove, EIdClockSettingsEdwin2);
	}

	Layout();
	if (aRedraw)
		DrawNow();
}

void CClockSettingsDialog::PostLayoutDynInitL()
{
	CEikChoiceList*		choiceList = (CEikChoiceList*) Control(EIdClockSettingsType);
	choiceList->SetCurrentItem(iClockSettings.iClockType);
	UpdateControlsL(EFalse);
}

TBool CClockSettingsDialog::ValidateDataL()
{
	CEikChoiceList*		choiceList = (CEikChoiceList*) Control(EIdClockSettingsType);
	if (choiceList->CurrentItem() == TClockSettings::EClockTypeConventional)
	{
		TBool ok = GetEdwinInt(iClockSettings.iConventionalNoOfMoves, EIdClockSettingsEdwin1);
		if (!ok || iClockSettings.iConventionalNoOfMoves <= 0) 
		{
			User::InfoPrint(_L("Invalid number of moves"));
			return EFalse;
		}

		CEikEdwin*	edwin = (CEikEdwin*) Control(EIdClockSettingsEdwin2);
		edwin->GetText(iClockSettings.iConventionalTime);
		TBuf8<16>	buf;
		buf.Copy(iClockSettings.iConventionalTime);

		if (!ParseTimeControl((char*) buf.PtrZ(), -1, iClockSettings.iConventionalNoOfMoves)) 
		{
			User::InfoPrint(_L("Invalid time"));
			return EFalse;
		}
	} 
	else 
	{
		TBool ok = GetEdwinInt(iClockSettings.iIncrementalAdditionalTimePerMove, EIdClockSettingsEdwin2);
		if (!ok || iClockSettings.iIncrementalAdditionalTimePerMove < 0) 
		{
			User::InfoPrint(_L("Invalid increment"));
			return EFalse;
		}

		CEikEdwin*	edwin = (CEikEdwin*) Control(EIdClockSettingsEdwin1);
		edwin->GetText(iClockSettings.iIncrementalInitialTime);
		TBuf8<16>	buf;
		buf.Copy(iClockSettings.iIncrementalInitialTime);
		if (!ParseTimeControl((char*)buf.PtrZ(), iClockSettings.iIncrementalAdditionalTimePerMove, 0)) 
		{
			User::InfoPrint(_L("Invalid initial time"));
			return EFalse;
		}
	}
	return ETrue;
}


TBool CClockSettingsDialog::OkToExitL(TInt aButtonId)
{
	TBool okToExit = ETrue;

	if (aButtonId == EEikBidOk)
	{
		okToExit = ValidateDataL();
	}
	else if (aButtonId == ECmdDefaults)
	{
		iClockSettings.Defaults();
		UpdateControlsL(ETrue);
		okToExit = EFalse;
	}

	return okToExit;
}

void CClockSettingsDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case EIdClockSettingsType:
		{
			CEikChoiceList*		choiceList = (CEikChoiceList*) Control(EIdClockSettingsType);
			iClockSettings.iClockType = choiceList->CurrentItem();
			UpdateControlsL(ETrue);
			break;
		}

		default:
			break;
	}
}

/**********************************************
 *
 * COneLineEdit dialog
 *
 **********************************************/

COneLineEdit::COneLineEdit(TDes& aLine) : iLine(aLine)
{
}

TBool COneLineEdit::OkToExitL(TInt aButtonId)
{
	if (aButtonId == EEikBidOk)
	{
		Text8FromEdwin();
		iLine.Copy(iText);		
	}
	return ETrue;
}

/**********************************************
 *
 * CAboutDialog dialog
 *
 **********************************************/


static void BitmapFileName(TDes& aFilename)
{
	CEikonEnv* env = CEikonEnv::Static();

		/*
		 * The .mbm file is stored in same directory as the .exe
		 * Note that I don't use CEikEnv::CreateBitmapL() which shares bitmaps if loaded 
		 * more than once (it calls CWsBitmap::Load with the sharable flag set).
		 */
	aFilename = ((CEikAppUi*)env->AppUi())->Application()->AppFullName();
	TInt		l = aFilename.Length();
	if (l > 3)
	{
		aFilename[l-1] = 'm';
		aFilename[l-2] = 'b';
		aFilename[l-3] = 'm';
	}
}

void CAboutDialog::PreLayoutDynInitL()
{
	TBuf<128>		text;
	TBuf<16>		versionText;
	TFileName		mbmFile;

#if (ECHESS_VERSION_PATCH > 0)
	versionText.Format(_L("%d.%d.%d"), ECHESS_VERSION_MAJOR, ECHESS_VERSION_MINOR, ECHESS_VERSION_PATCH);
#else
	versionText.Format(_L("%d.%d"), ECHESS_VERSION_MAJOR, ECHESS_VERSION_MINOR);
#endif

	HBufC*		versionFormat =	CEikonEnv::Static()->AllocReadResourceL(R_ABOUT_VERSION_TEXT);
	CleanupStack::PushL(versionFormat);

	const TDesC&	versionFormatDes = versionFormat->Des();
	text.Format(versionFormatDes, &versionText);

	CEikLabel*	label =(CEikLabel*)Control(ECtlAboutVersion);
	label->SetTextL(text);

	BitmapFileName(mbmFile);
	CEikImage* image= STATIC_CAST(CEikImage*,Control(ECtlAboutImage));
	image->CreatePictureFromFileL(mbmFile, EMbmEchessN33s, EMbmEchessN33w);

	CleanupStack::PopAndDestroy(); // 	versionFormat;
}



/**********************************************
 *
 * Edit text file dialog
 *
 **********************************************/

CEditTextFileDialog::CEditTextFileDialog(const TDesC& aFileName, const TDesC& aDefaultFileName)
	: iFileName(aFileName), iDefaultFileName(aDefaultFileName)
{
	ReadTextFile(iFileName);
}

void CEditTextFileDialog::ReadTextFile(const TDesC& aFileName, TBool aUpdateEdwin)
{
	iText8[0] = 0;
	TPtr8	ptr((TText8*)iText8, KMaxEditSize);
	RFile	file;
	TInt err = file.Open(iEikonEnv->FsSession(), aFileName, EFileRead);
	if (err == KErrNone)
	{
		err = file.Read(ptr, KMaxEditSize);
		if (err != KErrNone)
			iText8[0] = 0;
		file.Close();
	}
	Text8ToEdwin((TText8*)iText8, aUpdateEdwin);
}


TBool CEditTextFileDialog::StoreTextFile(const TDesC& aFileName)
{
	Text8FromEdwin();
	TPtrC8	ptr((TText8*)iText8, iText.Length());	// take length from the Unicode string

	iEikonEnv->FsSession().Delete(iFileName);
	RFile	file;
	TInt err = file.Create(iEikonEnv->FsSession(), aFileName, EFileWrite);
	if (err == KErrNone)
	{
		err = file.Write(ptr);
		file.Close();
	}
	return (err == KErrNone);
}


void CEditTextFileDialog::RunLD(const TDesC& aFileName, const TDesC& aDefaultFileName, TInt aTitleResId)
{
	CEikDialog*	dialog = new(ELeave)CEditTextFileDialog(aFileName, aDefaultFileName);
	dialog->PrepareLC(R_EDIT_FILE_DIALOG);
	dialog->SetTitleL(aTitleResId);
	dialog->RunLD();
}

TBool CEditTextFileDialog::OkToExitL(TInt aButtonId)
{
	TBool	okToExit = ETrue;

	switch (aButtonId)
	{
		case EBidFileEditDefault:
			User::InfoPrint(_L("Default settings"));
			ReadTextFile(iDefaultFileName, ETrue);	// read & update edwin
			okToExit = EFalse;
			break;

		case EEikBidOk:
			if (StoreTextFile(iFileName))
			{
				User::InfoPrint(_L("Restart EChess to use new settings"));
			}
			else
			{
				User::InfoPrint(_L("Could not save file"));
				okToExit = EFalse;
			}
		default:
			break;
	}
	return okToExit;
}

