/*******************************************************************
 *
 *	File:		EGnuChessView.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikappui.h>
#include <QuartzKeys.h>
#include <qikstockcontrols.hrh>
#include <eiklabel.h>
#include <eikmenub.h>

#include "EGnuChessView.h"
#include "EGnuChessApplication.h"
#include "EGnuChessModel.h"
#include "EGnuChessBoardCtl.h"
#include "EGnuChessUids.h"
#include <Echess.rsg>

static TBool sIsFlipChange = EFalse;

enum
{
	EScreenModeFlipOpen = 0,
	EScreenModeFlipClosed
};


/**************************************
 *
 * FO View
 *
 **************************************/

CEGnuChessViewBase::CEGnuChessViewBase(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl)
	: iEGnuChessViewCtl(aEGnuChessViewCtl), iModel(aModel)
{
}

CEGnuChessViewBase::~CEGnuChessViewBase()
{
}

void CEGnuChessViewBase::ViewActivatedL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/, const TDesC8& /*aCustomMessage*/)
{
	TPixelsAndRotation sizeAndRotation;
	CEikonEnv::Static()->ScreenDevice()->GetDefaultScreenSizeAndRotation(sizeAndRotation);
	CEikonEnv::Static()->ScreenDevice()->SetScreenSizeAndRotation(sizeAndRotation);
	iEGnuChessViewCtl.Activated();
}

void CEGnuChessViewBase::ViewDeactivated()
{
	iEGnuChessViewCtl.Deactivated();
	if (!pausing && !sIsFlipChange)
		PauseEvent();
	sIsFlipChange = EFalse;
}

void CEGnuChessViewBase::ViewConstructL()
{
}


CEGnuChessFOView::CEGnuChessFOView(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl)
	: CEGnuChessViewBase(aModel, aEGnuChessViewCtl)
{
}

CEGnuChessFOView::~CEGnuChessFOView()
{
}


TVwsViewId CEGnuChessFOView::ViewId() const
{
	return TVwsViewId(KUidEGnuChessApp, KUidChessFOView);
}

TVwsViewIdAndMessage CEGnuChessFOView::ViewScreenDeviceChangedL()
{
	sIsFlipChange = ETrue;
	return TVwsViewIdAndMessage(TVwsViewId(KUidEGnuChessApp, KUidChessFCView));
}

TBool CEGnuChessFOView::ViewScreenModeCompatible(TInt aScreenMode)
{
	return (aScreenMode == EScreenModeFlipOpen);
}

void CEGnuChessFOView::ViewActivatedL(const TVwsViewId& aPrevViewId, TUid aCustomMessageId, const TDesC8& aCustomMessage)
{
	CEGnuChessViewBase::ViewActivatedL(aPrevViewId, aCustomMessageId, aCustomMessage);
	CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);
	iEGnuChessViewCtl.ChangeLayout(EFalse);
}

/**************************************
 *
 * FC View
 *
 **************************************/

CEGnuChessFCView::CEGnuChessFCView(CEGnuChessModel& aModel, CEGnuChessViewCtl& aEGnuChessViewCtl)
	: CEGnuChessViewBase(aModel, aEGnuChessViewCtl)
{
}

CEGnuChessFCView::~CEGnuChessFCView()
{
}

TVwsViewId CEGnuChessFCView::ViewId() const
{
	return TVwsViewId(KUidEGnuChessApp, KUidChessFCView);
}

TVwsViewIdAndMessage CEGnuChessFCView::ViewScreenDeviceChangedL()
{
	sIsFlipChange = ETrue;
	return TVwsViewIdAndMessage(TVwsViewId(KUidEGnuChessApp, KUidChessFOView));
}

TBool CEGnuChessFCView::ViewScreenModeCompatible(TInt aScreenMode)
{
	return (aScreenMode == EScreenModeFlipClosed);
}

void CEGnuChessFCView::ViewActivatedL(const TVwsViewId& aPrevViewId, TUid aCustomMessageId, const TDesC8& aCustomMessage)
{
	CEGnuChessViewBase::ViewActivatedL(aPrevViewId, aCustomMessageId, aCustomMessage);
	CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(EFalse);
	iEGnuChessViewCtl.ChangeLayout(ETrue);
}


/**************************************
 *
 * View control
 *
 **************************************/


CEGnuChessViewCtl::CEGnuChessViewCtl(CEGnuChessModel& aModel): iModel(aModel)
{
}

CEGnuChessViewCtl::~CEGnuChessViewCtl()
{
	DestroyChildren();
}

CEGnuChessViewCtl* CEGnuChessViewCtl::NewL(CEGnuChessModel& aModel)
{
	CEGnuChessViewCtl*	self = new(ELeave)CEGnuChessViewCtl(aModel);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}

void CEGnuChessViewCtl::ConstructL()
{
	CCoeControlContainer::ConstructL();

	static_cast<CEikAppUi*>(iEikonEnv->AppUi())->AddToStackL(this);

	CreateWindowL();
	TRect rect = static_cast<CEikAppUi*>(iEikonEnv->AppUi())->ClientRect();
	SetExtent(rect.iTl, rect.Size());
	Window().SetBackgroundColor(iEikonEnv->Color(EColorControlBackground));

	iEGnuChessBoardControl = CEGnuChessBoardControl::NewL(iModel, *this);

	iWhiteTimeLabel = new(ELeave) CBitmappedLabel;
	iWhiteTimeLabel->SetContainerWindowL(*this);
	iWhiteTimeLabel->SetTextL(_L(" "));

	iBlackTimeLabel = new(ELeave) CBitmappedLabel;
	iBlackTimeLabel->SetContainerWindowL(*this);
	iBlackTimeLabel->SetTextL(_L(" "));

	iInfoLabel = new(ELeave) CBitmappedLabel;
	iInfoLabel->SetContainerWindowL(*this);
	iInfoLabel->SetTextL(_L(" "));

	AddChildL(iEGnuChessBoardControl);
	AddChildL(iWhiteTimeLabel);
	AddChildL(iBlackTimeLabel);
	AddChildL(iInfoLabel);

	MakeVisible(EFalse);
	ActivateL();
	ChangeLayout(EFalse);	// large layout

	iModel.AddModelGameObserver(*this);
	iModel.SetModelTimeObserver(*this);
	iModel.StartBackEnd();
}

void CEGnuChessViewCtl::Activated()
{
	if (iActivateCount == 0)
		MakeVisible(ETrue);
	iActivateCount++;
}

void CEGnuChessViewCtl::Deactivated()
{
	iActivateCount--;
	if (iActivateCount == 0)
		MakeVisible(EFalse);
}

_LIT(KUserFontName, "Courier");

void CEGnuChessViewCtl::ChangeLayout(TBool aSmall)
{
	iIsSmall = aSmall;
	TRect	rect(CEikonEnv::Static()->ScreenDevice()->SizeInPixels());
	if (!aSmall)
		rect = static_cast<CEikAppUi*>(iEikonEnv->AppUi())->ClientRect();
	SetRect(rect);
	if (aSmall)
	{
		iWhiteTimeLabel->UseFont(KUserFontName, 10);
		iBlackTimeLabel->UseFont(KUserFontName, 10);
	}
	else
	{
		iWhiteTimeLabel->ReleaseUserFont();
		iBlackTimeLabel->ReleaseUserFont();
	}

	LayoutChildren();

		// force clock update so that we display the correct labels
	DisplayBothClocks();
}


void CEGnuChessViewCtl::DoLayoutChildren()
{
	TInt	labelHeight = iInfoLabel->Font()->HeightInPixels();
	TInt	viewWidth = Size().iWidth;

	iEGnuChessBoardControl->SetBoardSize(iIsSmall);

	if (iIsSmall)
	{
			// chess board top left, no margins
		iEGnuChessBoardControl->SetPosition(TPoint(0, 0));
		TInt	x = iEGnuChessBoardControl->Rect().iBr.iX;
		TInt	y = iEGnuChessBoardControl->Rect().iBr.iY;
		TInt	width = Size().iWidth - x;
		iBlackTimeLabel->SetExtent(TPoint(x + 1, y - (labelHeight + 1)), TSize(width - 2, labelHeight));
		iWhiteTimeLabel->SetExtent(TPoint(x + 1, y - 2 * (labelHeight + 1) + 1), TSize(width - 2, labelHeight));
		TInt whiteClockY = iWhiteTimeLabel->Rect().iTl.iY - 1;
		iInfoLabel->SetExtent(TPoint(x + 1, 1), TSize(width - 2, whiteClockY - 2));
	}
	else
	{
			// top line is iInfoLabel with a 1 pixel margin all around
		iInfoLabel->SetExtent(TPoint(1, 1), TSize(viewWidth - 1, labelHeight));

			// underneath is the board (centered)
		TInt margin = Size().iWidth - iEGnuChessBoardControl->Size().iWidth;
		iEGnuChessBoardControl->SetPosition(TPoint(margin/2, labelHeight + 2));

			// bottom line the clocks, white left, black right
		TPoint	p(1, iEGnuChessBoardControl->Rect().iBr.iY + 1);
		TSize	s = iInfoLabel->Size();
		s.iWidth /= 2;
		iWhiteTimeLabel->SetExtent(p, s);
		iBlackTimeLabel->SetExtent(p + TPoint(s.iWidth,0) , s);
	}
}

void CEGnuChessViewCtl::HandleResourceChange(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);
	if(aType == KEikMessageColorSchemeChange )
	{
		Window().SetBackgroundColor(iEikonEnv->Color(EColorControlBackground));
	}
}

#include <qphoneappexternalinterface.h>

void CEGnuChessViewCtl::SwitchToStandbyView()
{
	TVwsViewId	viewId;
	viewId.iAppUid = KUidPhoneApp;
	CCoeAppUi*	coeAppUi = (CCoeAppUi*) ControlEnv()->AppUi();
	if (coeAppUi->GetDefaultViewId(viewId) == KErrNone)
	{
		TRAPD(err, coeAppUi->ActivateViewL(viewId));
		if (!err)
		{
			iCoeEnv->RootWin().SetOrdinalPosition(-1);
		}
	}
}


TKeyResponse CEGnuChessViewCtl::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	TKeyResponse keyResponse = EKeyWasNotConsumed;

	if (iIsSmall)
	{
		switch (aType)
		{
			case EEventKey:
				if (aKeyEvent.iRepeats == 0)
				{
					if (aKeyEvent.iCode == EKeyDeviceA)
					{
						keyResponse = EKeyWasConsumed;
						PauseEvent();
					}
				}
				break;

			case EEventKeyDown:
				if (aKeyEvent.iScanCode == EStdKeyDevice3)
				{
					keyResponse = EKeyWasConsumed;
					SwitchToStandbyView();
				}
				break;
		}
	}

	return keyResponse;
}


void CEGnuChessViewCtl::DisplayMessageL(const TDesC& aMessage)
{
	iInfoLabel->SetTextL(aMessage);
	DrawChildNow(iInfoLabel);
}

void CEGnuChessViewCtl::UpdateTimeClockL(TBool aWhite, const TDesC& aTimeString, TBool /*aHighlight*/)
{
	if (aWhite)
	{
		_LIT(KColorName, "White");
		_LIT(KSmallColorName, "W");
		iTimeText.Format(_L("%S:%S"), iIsSmall ? &KSmallColorName : &KColorName, &aTimeString);
		iWhiteTimeLabel->SetTextL(iTimeText);
		DrawChildNow(iWhiteTimeLabel);
	}
	else
	{
		_LIT(KColorName, "Black");
		_LIT(KSmallColorName, "B");
		iTimeText.Format(_L("%S:%S"), iIsSmall ? &KSmallColorName : &KColorName, &aTimeString);
		iBlackTimeLabel->SetTextL(iTimeText);
		DrawChildNow(iBlackTimeLabel);
	}
}


void CEGnuChessViewCtl::SetBoardSettings(const TBoardSettings& aBoardSettings)
{
	iEGnuChessBoardControl->SetBoardSettings(aBoardSettings);
	iEGnuChessBoardControl->DrawNow();
}
