#include <stdarg.h>
#include <estlib.h>
#include <stdlib.h>
#include <f32file.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys\fcntl.h>
#include <sys\types.h>
#include <sys\socket.h>
#include <netinet\in.h>

#include "debug.h"
#include "ChessEngine.h"

static TTimeIntervalMicroSeconds32	sTickPeriod;


extern "C"
{
void __gccmain (void) {}
int main (int argc, char *argv[]);
typedef enum {cpu, elapsed, microseconds} TIME_TYPE;
unsigned int ReadClock(TIME_TYPE type);
}


unsigned int ReadClock(TIME_TYPE type)
{
	unsigned int clock;
	switch (type)
	{
		case microseconds:
		case cpu:
			clock = User::TickCount() * sTickPeriod.Int();
			break;

		case elapsed:		/* in ms */
		default:
			clock = (User::TickCount() * sTickPeriod.Int()) / 1000;
			break;
	}
	return clock;
}	


void RDebugPrint(const char* format, ...)
{
	TBuf<128>	buf;
	char	text[128];
	va_list     argptr;

	va_start(argptr, format);
	vsprintf(text, format, argptr);
	va_end(argptr);

	TPtrC8	ptr((TText8*) text);
	buf.Copy(ptr);
	RDebug::Print(_L("%S"), &buf);
	RDebug::Print(_L("%S"), &buf);
}




#define STD_ERR_FILE "c:\\Documents\\Echess\\chesseng.err"

TInt ConnectToServer()
{
	struct sockaddr_in	serv_addr;
	memset(&serv_addr, 0, sizeof(serv_addr));
	IN_SET_LOOPBACK_ADDR(&serv_addr);
	serv_addr.sin_port = htons(6543);

	DPRINT((HERE("CHESS ENGINE: ConnectToServer")));

	// create socket
	TInt success = 0;
	TInt sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock>=0)
	{
		// try a couple of times to connect
		TInt err = -1;
		for (TInt i=0 ; (err<0) && (i<10) ; i++)
		{
			err = connect(sock, (struct sockaddr*) &serv_addr, sizeof(serv_addr));
			if (err < 0)
				User::After(500 * 1000);
		}
		DPRINT((HERE("CHESS ENGINE: ConnectToServer - 1")));
		if (err>=0)
		{
				/* 
				 * close stdin, stdout and stderr
				 * map the socket connection to the front end on stdin and stdout
				 * create file for stderr messages
				 */
			close(0);
			close(1);
			close(2);
			err = dup2(sock, 0);
			if (err>=0)
			{
				err = dup2(sock, 1);
				if (err>=0)
				{
					int fd = creat(STD_ERR_FILE, 0);
					if (fd>=0)
						dup2(fd, 2);
					success = 1;
				}
				else
				{
					DPRINT((HERE("ERROR: dup2 on fd=1 failed, error=%d\n"), err));
				}
			}
			else
			{
				DPRINT((HERE("ERROR: dup2 on fd=0 failed, error=%d\n"), err));
			}
		}
		else
		{
			DPRINT((HERE("ERROR: socket failed, error=%d\n"), sock));
			close(sock);
			sock = -1;
		}
	}
	else
	{
		DPRINT((HERE("ERROR: socket failed, error=%d\n"), sock));
	}
	DPRINT((HERE("CHESS ENGINE: ConnectToServer done")));

	return success;
}



TInt SignalThreadFunc(TAny* /*aAny*/)
{
	RSemaphore	sem;
	if (sem.OpenGlobal(_L("CHESS_SIG")) == KErrNone)
	{
		while(ETrue)
		{
			sem.Wait();
			DPRINT((HERE("CHESS ENGINE: we've been signalled\n")));
			ChessEngineSignal();		// implemented in chess engine specific code
		}
	}
	return 0;
}

static void StartSignalThread()
{
	RThread		t;
	t.Create(_L("XXX"), SignalThreadFunc, 2*1024, NULL, NULL);
	t.Resume();
	t.Close();
}


TBool RunningFromExternalDrive()
{
	RProcess	me;
	TChar drive = me.FileName()[0];
	TBool external = ((drive == TChar('D')) || (drive == TChar('d')));
	return external;
}

void ChessEngine(void)
{
//	TBool	external = RunningFromExternalDrive();

	setvbuf(stdout, NULL, _IONBF, 0);
	setvbuf(stdin, NULL, _IONBF, 0);
	if (ConnectToServer())
	{
		DPRINT((HERE("CHESS ENGINE: client successfully connected\n")));

		mkdir("c:\\Documents", 0);
		mkdir("c:\\Documents\\echess", 0);
		chdir("c:\\Documents\\echess");

		StartSignalThread();
		DPRINT((HERE("CHESS ENGINE: calling ChessEngineMain\n")));
		ChessEngineMain();		// implemented in chess engine specific code
	}
}


int E32Main(void)
{
	DPRINT((HERE("CHESS ENGINE: E32Main")));
	UserHal::TickPeriod(sTickPeriod);
	srand(User::TickCount());
	ChessEngine();
	return 0;
}


#ifdef __WINS__

#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
EXPORT_C TInt WinsMain()
{
#if defined(__WINS__)
	::SetThreadPriority(::GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);
	E32Main();
#endif
	return KErrNone;
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}
#endif



//#define PROFILE_ALLOCS 1
//#define BREAK_ON_BIG_ALLOCS 1
#undef BREAK_ON_BIG_ALLOCS
#ifdef PROFILE_ALLOCS

#include <stdlib.h>

void DumpHeap()
{
	TInt cells = User::CountAllocCells();
	TInt mem;
	User::AllocSize(mem);
	RDebug::Print(_L("  HEAP: cels=%d, size=%d KB"), cells, mem/1024);
}

static size_t tot_alloc = 0;
void* malloc(size_t s)
{
#ifdef BREAK_ON_BIG_ALLOCS
#ifdef __WINS__
	if (s> 512*1024)
		__asm int 3;
#endif
#endif
	void*	result;
	result = User::Alloc(s);
	if (!result)
	{
		User::InfoPrint(_L("no memory"));
		User::After(3 * 1000 * 1000);
	}
	tot_alloc += s;
	RDebug::Print(_L("my_malloc, size=%d, tot_alloc=%d"), s, tot_alloc);
	DumpHeap();
	return result;
}

void* calloc(size_t num, size_t size)
{
#ifdef BREAK_ON_BIG_ALLOCS
#ifdef __WINS__
	if (size*num> 512*1024)
		__asm int 3;
#endif
#endif
	void*	result;
	result = User::Alloc(num * size);
	if (result)
		Mem::FillZ(result, num*size);
	if (!result)
	{
		User::InfoPrint(_L("no memory"));
		User::After(3 * 1000 * 1000);
	}
	tot_alloc += num*size;
	RDebug::Print(_L("my_calloc, size=%d, tot_alloc=%d"), num*size, tot_alloc);
	DumpHeap();
	return result;
}

void *realloc( void *memblock, size_t size )
{
#ifdef BREAK_ON_BIG_ALLOCS
#ifdef __WINS__
	if (size> 512*1024)
		__asm int 3;
#endif
#endif
	TInt	oldSize = memblock ? User::AllocLen(memblock) : 0;

	void*	result = User::ReAlloc(memblock, size);
	if (result)
		Mem::FillZ(result, size);
	if (!result)
	{
		User::InfoPrint(_L("no memory"));
		User::After(3 * 1000 * 1000);
	}
	tot_alloc += size - oldSize;
	RDebug::Print(_L("my_realloc, size=%d (old=%d), tot_alloc=%d"), size, oldSize, tot_alloc);
	DumpHeap();
	return result;
}

void free( void *memblock)
{
	TInt	size = memblock ? User::AllocLen(memblock) : 0;

	User::Free(memblock);
	tot_alloc -= size;
	RDebug::Print(_L("free, size=%d, tot_alloc=%d"), size, tot_alloc);
	DumpHeap();
}


#endif
